#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import json


def make_json_callback_return(
        data=None,
        callback: str = None
):
    from flask import jsonify
    """
    判断包装返回，用以兼容ajax的jsonp请求，以实现支持前端跨域
    """
    if callback is None:
        # 无callback参数，正常返回json
        return jsonify(data)
    else:
        # 有callback参数，拼接字符串返回
        if type(data) == 'json':
            return '%s(%s)' % (callback, json.dumps(data))
        else:
            return '%s(%s)' % (callback, data)



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
from collections import OrderedDict
from . import lazydict
import openpyxl  # 用于处理.xlsx，文档：https://openpyxl.readthedocs.io/en/stable/
import datetime
import xlrd  # 用于处理.xls
import os
from .lazypath import path_separator


# -------------------------------读取功能区域-------------------------------
def read_xls(
        file: str,
        sheet_name: str = None,
        date_cols: dict = None,
        auto_conv_date: bool = False  # 默认格式：'%Y-%m-%d %H:%M:%S'
) -> dict:
    """
    读取xls表格
    :param file: 文件路径
    :param sheet_name: 指定sheet名称，读取指定的sheet
    :param date_cols: 按照既定规则转换时间，例如：{'日期': '%Y-%m-%d','时间': '%H:%M:%S'}，就会将表中对应字段转换为对应格式的字符串
    :param auto_conv_date: 自动转换，会将时间类型的数据自动转换为默认格式的字符串

    :returns: 读取结果字典，字典的第一层key为sheet_name

    cell(a,b).ctype 值类型
    0 empty,
    1 string,
    2 number,
    3 date,
    4 boolean,
    5 error
    """
    if date_cols is None:
        data_cols_cols = []
    else:
        data_cols_cols = date_cols.keys()
    table_data_dict = OrderedDict()  # 有序字典
    table_data = xlrd.open_workbook(file)  # 打开表
    for each_sheet in table_data.sheets():  # 遍历所有sheet
        if sheet_name is None:
            pass
        else:
            if each_sheet.name == sheet_name:  # 判断当前的sheet是否是需要获取数据的sheet
                pass
            else:
                continue
        row_num = each_sheet.nrows  # 获取行数
        col_num = each_sheet.ncols  # 获取列数
        sheet_data_list = list()
        for each_row in range(1, row_num):  # 遍历每一行
            each_row_data = dict()
            for each_col in range(col_num):  # 对于当前行，遍历每一列，将每一行的数据以dict的形式组织到each_row_data里面
                col_name = each_sheet.cell(0, each_col).value  # 列名
                if col_name is None:
                    continue
                else:
                    pass
                col_value = each_sheet.cell(each_row, each_col).value  # 值
                col_type = each_sheet.cell(each_row, each_col).ctype  # 值类型
                if col_value is None:
                    pass
                else:
                    if col_type == 3:
                        if col_name in data_cols_cols:
                            col_value = xlrd.xldate_as_datetime(col_value, 0).strftime(date_cols[col_name])  # 转换为datatime后转换为需要的格式
                        else:
                            if len(data_cols_cols) == 0 and auto_conv_date is True:
                                col_value = xlrd.xldate_as_datetime(col_value, 0).strftime('%Y-%m-%d %H:%M:%S')
                            else:
                                pass
                    else:
                        pass
                each_row_data[col_name] = col_value
            each_row_data_single = list(set(each_row_data.values()))
            if len(each_row_data_single) == 1:
                if each_row_data_single[0] is None:
                    continue  # 去除全空值
                else:
                    pass
            else:
                pass
            sheet_data_list.append(each_row_data)  # 将每一行的数据dict组织到当前sheet的数据list中
        table_data_dict[each_sheet.name] = sheet_data_list  # 将每个sheet的数据组织到table_data_dict里面
    return table_data_dict


def read_xlsx(
        file: str,
        sheet_name: str = None,
        date_cols: dict = None,
        auto_conv_date: bool = False  # 默认格式：'%Y-%m-%d %H:%M:%S'
) -> dict:
    """
    读取xlsx表格
    :param file: 文件路径
    :param sheet_name: 指定sheet名称，读取指定的sheet
    :param date_cols: 按照既定规则转换时间，例如：{'日期': '%Y-%m-%d','时间': '%H:%M:%S'}，就会将表中对应字段转换为对应格式的字符串
    :param auto_conv_date: 自动转换，会将时间类型的数据自动转换为默认格式的字符串

    :returns: 读取结果字典，字典的第一层key为sheet_name

    cell(a,b).data_type 值类型
    n None
    s string
    d date
    其他暂时未知
    """
    if date_cols is None:
        data_cols_cols = []
    else:
        data_cols_cols = date_cols.keys()
    table_data_dict = OrderedDict()  # 有序字典
    table_data = openpyxl.load_workbook(file)  # 打开表
    for each_sheet in table_data.worksheets:  # 遍历所有sheet
        if sheet_name is None:
            pass
        else:
            if each_sheet.title == sheet_name:  # 判断当前的sheet是否是需要获取数据的sheet
                pass
            else:
                continue
        row_num = each_sheet.max_row  # 获取行数
        col_num = each_sheet.max_column  # 获取列数
        sheet_data_list = list()
        for each_row in range(1, row_num):  # 遍历每一行
            each_row_data = dict()
            for each_col in range(1, col_num + 1):  # 对于当前行，遍历每一列，将每一行的数据以dict的形式组织到each_row_data里面
                col_name = each_sheet.cell(1, each_col).value
                if col_name is None:
                    continue
                else:
                    pass
                col_value = each_sheet.cell(each_row + 1, each_col).value  # 原来为日期的会自动转换
                col_type = each_sheet.cell(each_row + 1, each_col).data_type
                if col_value is None:
                    pass
                else:
                    if col_type == 'd':
                        if len(data_cols_cols) == 0 and auto_conv_date is True:
                            col_value = col_value.strftime('%Y-%m-%d %H:%M:%S')
                        else:
                            if col_name in data_cols_cols:
                                col_value = col_value.strftime(date_cols[col_name])  # 已经转换的直接按照目标格式转换
                            else:
                                pass
                    else:
                        pass
                each_row_data[col_name] = col_value
            each_row_data_single = list(set(each_row_data.values()))
            if len(each_row_data_single) == 1:
                if each_row_data_single[0] is None:
                    continue  # 去除全空值
                else:
                    pass
            else:
                pass
            sheet_data_list.append(each_row_data)  # 将每一行的数据dict组织到当前sheet的数据list中
        table_data_dict[each_sheet.title] = sheet_data_list  # 将每个sheet的数据组织到table_data_dict里面
    return table_data_dict


def read(
        file: str,
        sheet_name: str = None,
        date_cols: dict = None,  # date转换为字符串
        auto_conv_date: bool = False  # 默认格式：'%Y-%m-%d %H:%M:%S'
) -> dict:
    """
    读取表格数据，支持xls和xlsx两种格式，会自动判断
    :param file: 读取指定文件
    :param sheet_name: 读取指定sheet，一旦指定，将只读取这个sheet的内容
    :param date_cols: 按照既定规则转换时间，例如：{'日期': '%Y-%m-%d','时间': '%H:%M:%S'}，就会将表中对应字段转换为对应格式的字符串
    :param auto_conv_date: 自动转换，会将时间类型的数据自动转换为默认格式的字符串

    :return: 表格中的所有数据，以sheet的名称为key的dict（有序），每个value是由值字典组成的list
    """
    suffix = file.split('.')[-1]  # 获取文件后缀名
    if suffix == 'xlsx':
        table_data_dict = read_xlsx(
            file=file,
            sheet_name=sheet_name,
            date_cols=date_cols,
            auto_conv_date=auto_conv_date
        )
        return table_data_dict
    elif suffix == 'xls':
        table_data_dict = read_xls(
            file=file,
            sheet_name=sheet_name,
            date_cols=date_cols,
            auto_conv_date=auto_conv_date
        )
        return table_data_dict
    else:
        return OrderedDict()


# -------------------------------读取功能区域-------------------------------
# -------------------------------写入功能区域-------------------------------


def save_xlsx(
        file: str,
        value: dict = None,
        date_cols: dict = None,
        col_name_dict: dict = None
):
    """
    如果输入的value是乱序，将重新排序
    :param file: 文件路径
    :param value: 需要保存的数据
    :param date_cols: 按照既定规则转换时间，例如：{'日期': '%Y-%m-%d','时间': '%H:%M:%S'}，就会将表中对应字段转换为对应格式的时间
    :param col_name_dict: 自定义列名的对照关系，规则为：{'旧名称1':'新名称1', '旧名称2':'新名称2'}
    将输出保存后的文件绝对路径
    """
    if os.path.isabs(file):  # 判断是否为绝对路径
        pass
    else:
        file = os.getcwd() + path_separator + file
    if value is None:
        return
    else:
        wb = openpyxl.Workbook()
        sheet_index = 0
        for sheet_name, sheet_data in value.items():
            sheet = wb.create_sheet(title=sheet_name, index=sheet_index)
            row_num = 1  # 行序号
            col_num = 1  # 列序号
            if len(sheet_data) > 0:
                # 如果数据是乱序，需要先排序
                sheet_data_f = lazydict.list_same_order_dict(list_data=sheet_data)
                if date_cols is not None:
                    for each_sheet_data_f in sheet_data_f:
                        for date_key, date_f in date_cols.items():
                            try:
                                date_value = each_sheet_data_f.get(date_key)
                                if date_value is not None:
                                    date_value_f = datetime.datetime.strptime(date_value, date_f)
                                    each_sheet_data_f[date_key] = date_value_f
                                else:
                                    pass
                            except:
                                pass
                else:
                    pass
                # 写入标题行
                for key in sheet_data_f[0]:
                    if col_name_dict is not None and col_name_dict.get(key) is not None:
                        sheet.cell(
                            row=row_num,
                            column=col_num,
                            value=col_name_dict.get(key)
                        )  # 写入标题行
                    else:
                        sheet.cell(
                            row=row_num,
                            column=col_num,
                            value=key
                        )  # 写入标题行
                    col_num += 1

                # 写入数据行
                row_num += 1
                for each in sheet_data_f:
                    col_num = 1
                    for key in each:
                        sheet.cell(
                            row=row_num,
                            column=col_num,
                            value=each.get(key)
                        )
                        col_num += 1
                    row_num += 1
            else:
                continue
            sheet_index += 1
        wb.remove(wb.worksheets[-1])  # 删除最后一个默认的sheet
        wb.save(file)
        return file


def save(
        file: str,
        value: dict = None,
        date_cols: dict = None,  # 字符串转换为date
        overwrite: bool = False,
        col_name_dict: dict = None
):
    """
    如果输入的value是乱序，将重新排序
    :param file: 文件路径，支持：xlsx
    :param value: 需要保存的数据
    :param date_cols: 按照既定规则转换时间，例如：{'日期': '%Y-%m-%d','时间': '%H:%M:%S'}，就会将表中对应字段转换为对应格式的时间
    :param overwrite: 是否覆盖
    :param col_name_dict: 自定义列名的对照关系，规则为：{'旧名称1':'新名称1', '旧名称2':'新名称2'}
    """
    if overwrite is False:
        if os.path.exists(file) is True:
            return
        else:
            pass
    else:
        pass

    suffix = file.split('.')[-1]  # 获取文件后缀名
    if suffix == 'xlsx':
        save_xlsx(
            file=file,
            value=value,
            date_cols=date_cols,
            col_name_dict=col_name_dict
        )
        return
    elif suffix == 'xls':
        print(':( xls is not supported')
        return
    else:
        print(':( file is not supported')
        return



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
from collections import OrderedDict
from urllib import parse


def dict_list_ranker_x(dict_list, rank_by_list):
    """
    对dict组成的list按照某个元素重新排序
    可对多个元素排序,目前支持2个
    rank_by_list：[['a','asc']]
    :return:
    """
    ranker_res_list = list()
    for each_dict in dict_list:
        each_rank_by_value_1 = each_dict.get(rank_by_list[0][0])
        each_rank_by_value_2 = each_dict.get(rank_by_list[1][0])
        if len(ranker_res_list) == 0:
            ranker_res_list.append(each_dict)
        else:
            pos_num = 0
            for each_ranker_res in ranker_res_list:
                each_ranker_res_rank_by_value_1 = each_ranker_res.get(rank_by_list[0][0])
                each_ranker_res_rank_by_value_2 = each_ranker_res.get(rank_by_list[1][0])
                if rank_by_list[0][1].lower() == 'asc' and rank_by_list[1][1].lower() == 'desc':
                    if each_rank_by_value_1 > each_ranker_res_rank_by_value_1:
                        pos_num += 1
                    elif each_rank_by_value_1 == each_ranker_res_rank_by_value_1:
                        if each_rank_by_value_2 < each_ranker_res_rank_by_value_2:
                            pos_num += 1
                        else:
                            pass
                    elif each_rank_by_value_1 < each_ranker_res_rank_by_value_1:
                        pass
            ranker_res_list.insert(pos_num, each_dict)
    return ranker_res_list


def dict_list_ranker(dict_list, rank_by_list):
    """
    对dict组成的list按照某个元素重新排序
    可对多个元素排序,目前支持1个
    rank_by_list：[['a','asc']]
    :return:
    """
    ranker_res_list = list()
    for each_dict in dict_list:
        each_rank_by_value_1 = each_dict.get(rank_by_list[0][0])
        if len(ranker_res_list) == 0:  # 如果结果列表为空，放入初始值
            ranker_res_list.append(each_dict)
        else:
            pos_num = 0
            for each_ranker_res in ranker_res_list:
                each_ranker_res_rank_by_value_1 = each_ranker_res.get(rank_by_list[0][0])
                if each_rank_by_value_1 > each_ranker_res_rank_by_value_1:
                    pos_num += 1
                else:
                    pass
            ranker_res_list.insert(pos_num, each_dict)
    if rank_by_list[0][1].lower() == 'asc':
        return ranker_res_list
    else:
        ranker_res_list.reverse()
        return ranker_res_list


def dict_list_group(list_in, by):
    """
    将dict组成的list按照某个key的值分组，分组后组成新的dict嵌套list
    :param list_in:
    :param by:
    :return:
    """
    by_list = list()
    for each_dict in list_in:
        by_value = each_dict.get(by)
        by_list.append(by_value)
    by_set = set(by_list)
    group_dict = dict()
    for each_by in by_set:
        group_dict[each_by] = list()
    for each_dict in list_in:
        by_value = each_dict.get(by)
        group_dict[by_value].extend([each_dict])
    return group_dict


def dict_key_f(in_dict):
    """
    将以驼峰方式命名的dict转换为下划线方式
    """
    out_dict = dict()
    for key, value in in_dict.items():
        # key_split = list(key)
        # print(key, key_split)
        key_split_new = list()
        for index, temp_key in enumerate(key):
            if temp_key.isupper() is True:
                if index == 0:
                    key_split_new.append(temp_key.lower())
                else:
                    key_split_new.append('_')
                    key_split_new.append(temp_key.lower())
            else:
                key_split_new.append(temp_key)
        key_new = ''.join(key_split_new)
        # print(key_new, key_split_new)
        out_dict[key_new] = value
    return out_dict


def list_dict_key_f(in_list):
    """
    将以驼峰方式命名的list嵌套dict转换为下划线方式
    """
    out_list = list()
    for each_dict in in_list:
        out_dict = dict_key_f(each_dict)
        out_list.append(out_dict)
    return out_list


def dict_value_parse(dict_in):
    # 对字典的值进行解码
    dict_out = dict()
    for key, value in dict_in.items():
        if isinstance(value, str):
            dict_out[key] = parse.unquote(value)
        else:
            dict_out[key] = value
    return dict_out


def list_dict_value_parse(list_in):
    # 在dict上面封装一层list，解码
    list_out = list()
    for each_list_in in list_in:
        dict_out = dict_value_parse(dict_in=each_list_in)
        list_out.append(dict_out)
    return list_out


def dict_tiler(dict_in, connector_str='-'):
    """
    字典展开器
    只展开一层
    将嵌套了一层的dict铺展开，键值之间用"-"连接
    针对纯dict形式
    :param dict_in: 输入一个待处理的dict
    :param connector_str: 输入连接符
    :return: 输出处理好的dict
    """
    dict_f = dict()
    if dict_in is None:
        return
    elif len(dict_in) == 0:
        return
    else:
        for key in dict_in:
            value = dict_in.get(key)
            if isinstance(value, dict):  # 如果某个key的值是dict，则展开这组
                for each_value_key in value:
                    dict_f['%s%s%s' % (key, connector_str, each_value_key)] = value.get(each_value_key)
            else:
                dict_f[key] = value
        return dict_f


def list_dict_tiler(list_in, connector_str='-'):
    list_out = list()
    for each in list_in:
        dict_f = dict_tiler(dict_in=each, connector_str=connector_str)
        list_out.append(dict_f)
    return list_out


def list_same_order_dict(
        list_data: list = None,
        na_value=None
) -> list:
    """
    list重排格式化，将单层dict组成的list按照全部key的顺序排序并补充缺失值
    :param list_data: 原始list
    :param na_value: 缺失值的默认值
    :return list_data_f: 处理后的结果

    test data:
        [
            {'a': 1},
            {'b': 2},
            {'b': 2, 'a': 1},
            {'c': 21}
        ]
    """
    keys = list()
    for each_dict in list_data:
        each_keys = list(each_dict.keys())
        for each_key in each_keys:
            if each_key in keys:
                continue
            else:
                keys.append(each_key)
    list_data_f = list()
    for each_dict in list_data:
        each_dict_order = OrderedDict()  # 有序字典
        for each_key in keys:
            each_dict_order[each_key] = each_dict.get(each_key, na_value)
        list_data_f.append(each_dict_order)
    return list_data_f



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import showlog
import redis
import time
import envx

default_db = 0  # 默认数据库
default_env_file_name = 'local.redis.env'  # 默认链接信息文件
"""
统一定义：
数据库层为db
集合层叫name
下一层hash的叫key,value

全局默认解码(decode_responses=True)

redis的基本用法可以参考：https://www.runoob.com/redis

python的redis用法可以参考：https://www.runoob.com/w3cnote/python-redis-intro.html

返回：
    {
        "code": 0,
        "msg": "ok",
        "data": <返回的数据>/None
    }
    
    code=0为正常，code!=0为异常，msg参数为原因
    
    code=1为未知错误
"""


class Basics:
    def __init__(
            self,
            con_info: dict = None,  # 连结信息，如果设置，将优先使用
            db: int = None,  # 需要连接的数据库，以数字为序号的，从0开始
            host=None,  # 连接的域名
            port=None,  # 连接的端口
            password=None,  # 连接的密码,
            max_connections=None,
            decode_responses=True
    ):
        # 初始化所有参数
        if con_info is not None:
            self.con_db = con_info.get('db', 0)
            self.host = con_info.get('host', 'localhost')
            self.port = con_info.get('port', 6379)
            self.pwd = con_info.get('password')
            self.max_connections = con_info.get('max_connections')
            self.decode_responses = con_info.get('decode_responses', True)
        else:
            if db is None:
                self.con_db = 0
            else:
                self.con_db = db
            self.host = host
            self.port = port
            self.pwd = password
            self.max_connections = max_connections
            self.decode_responses = decode_responses
        self.pool = self.make_connect_pool()
        self.conn = self.connect()

    def make_connect_pool(
            self
    ):
        # 使用连接池连接，节省每次连接用的时间
        pool = redis.ConnectionPool(
            host=self.host,
            port=self.port,
            password=self.pwd,
            db=self.con_db,
            max_connections=self.max_connections,
            decode_responses=self.decode_responses
        )
        return pool

    def connect(
            self
    ):
        # 从连接池中拿出一个连接
        connection = redis.Redis(
            connection_pool=self.pool
        )
        return connection

    def delete_key(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        删除 db中的一个key
        data: 存在key且删除成功为1，不存在key/删除失败为0
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.delete(key)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def get_db_key_list(
            self,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        读取 db中键列表
        若有key，则返回key列表；若无key，则返回空列表
        """
        while True:
            try:
                inner_keys = list(self.conn.keys())
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": inner_keys
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_get_values_count(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        获取 list 的 元素数量
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.llen(key)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_read_key_values(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        读取 列表的 键的所有值列表
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.lrange(name=key, start=0, end=-1)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_read_first_value(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        读取 列表的 第一个元素
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.lindex(key, 0)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_read_last_value(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        获取 列表的 最后一个元素
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.lindex(key, -1)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_add_l(
            self,
            key,
            value,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        在list左侧添加值，作为队列使用时一般不用此方法，一般在右侧入队列，在左侧出队列
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.lpush(key, value)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_add_r(
            self,
            key,
            value,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        在list右侧添加值，作为队列使用时，一般使用此方法从右侧入队列
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.rpush(key, value)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_pop_l(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        从左侧出队列，作为队列时，经常使用此方法
        """
        while True:
            try:
                key_type = self.conn.type(key)
                if key_type == 'list':
                    return {
                        "code": 0,
                        "msg": "ok",
                        "data": self.conn.lpop(key)
                    }
                else:
                    return {
                        "code": 2,
                        "msg": f'key type is not list, type is {key_type}',
                        "data": None
                    }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def list_pop_r(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        从右侧侧出队列
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.rpop(key)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def set_save(
            self,
            name,
            value,
            ex=None,
            px=None,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        # 设置键值，ex过期时间（秒），px过期时间（毫秒）
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.set(
                        name=name,
                        value=value,
                        ex=ex,
                        px=px,
                        nx=False,
                        xx=False
                    )
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def set_get(
            self,
            name,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        # 获取键值
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.get(name)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def count_add(
            self,
            name,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Incr 命令将 key 中储存的数字值增一。
        如果 key 不存在，那么 key 的值会先被初始化为 0 ，然后再执行 INCR 操作。
        如果值包含错误的类型，或字符串类型的值不能表示为数字，那么返回一个错误。
        本操作的值限制在 64 位(bit)有符号数字表示之内。
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.incr(name=name)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def count_reduce(
            self,
            name,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Decr 命令将 key 中储存的数字值减一。
        如果 key 不存在，那么 key 的值会先被初始化为 0 ，然后再执行 DECR 操作。
        如果值包含错误的类型，或字符串类型的值不能表示为数字，那么返回一个错误。
        本操作的值限制在 64 位(bit)有符号数字表示之内。
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.decr(name=name)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def count_read(
            self,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        # 键 计数 获取值
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.get(key)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    # -------------------- hash --------------------

    def hash_set(
            self,
            name,
            key,
            value,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Hset 命令用于为哈希表中的字段赋值 。
        如果哈希表不存在，一个新的哈希表被创建并进行 HSET 操作。
        如果字段已经存在于哈希表中，旧值将被覆盖。
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.hset(
                        name=name,
                        key=key,
                        value=value
                    )
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def hash_delete(
            self,
            name,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Hdel 命令用于删除哈希表 key 中的一个或多个指定字段，不存在的字段将被忽略。
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.hdel(
                        name,
                        key
                    )
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def hash_keys(
            self,
            name,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Hkeys 命令用于获取哈希表中的所有域（field）。
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.hkeys(name=name)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def hash_get(
            self,
            name,
            key,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Hget 命令用于返回哈希表中指定字段的值。
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.hget(name=name, key=key)
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def hash_get_many(
            self,
            name,
            key_list: list,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Hmget 命令用于返回哈希表中，一个或多个给定字段的值。
        如果指定的字段不存在于哈希表，那么返回一个 nil 值。
        """
        while True:
            try:
                return {
                    "code": 0,
                    "msg": "ok",
                    "data": self.conn.hmget(
                        name=name,
                        keys=key_list
                    )
                }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }

    def hash_get_all(
            self,
            name,
            auto_reconnect: bool = True,  # 自动重连
            reconnect_delay: int = 1  # 重连延时，单位为秒
    ):
        """
        Redis Hgetall 命令用于返回哈希表中，所有的字段和值。
        在返回值里，紧跟每个字段名(field name)之后是字段的值(value)，所以返回值的长度是哈希表大小的两倍。
        """
        while True:
            try:
                key_type = self.conn.type(name=name)
                if key_type is None:
                    return {
                        "code": 1,
                        "msg": "type is None",
                        "data": None
                    }
                elif 'hash' not in key_type:
                    return {
                        "code": 1,
                        "msg": "type is not hash",
                        "data": None
                    }
                else:
                    return {
                        "code": 0,
                        "msg": "ok",
                        "data": self.conn.hgetall(name=name)
                    }
            except redis.exceptions.ConnectionError as e:
                if auto_reconnect is True:
                    showlog.warning(f'连接失败，将在{reconnect_delay}秒后重连...')
                    time.sleep(1)
                    self.conn = self.connect()
                else:
                    return {
                        "code": 1,
                        "msg": e,
                        "data": None
                    }


def make_con_info(
        env_file_name: str = default_env_file_name
):
    """
    制作连接信息
    :param env_file_name: 环境文件名，默认为

    """
    # ---------------- 固定设置 ----------------
    inner_env = envx.read(file_name=env_file_name)
    if inner_env is None or len(inner_env) == 0:
        showlog.warning('[%s]文件不存在或文件填写错误！' % env_file_name)
        exit()
    else:
        max_connections = inner_env.get('max_connections')
        if max_connections is not None:
            try:
                max_connections = int(max_connections)
            except:
                showlog.warning('max_connections必须是数字！')
        else:
            max_connections = None
        con_info = {
            "host": inner_env.get('host', 'localhost'),
            "port": int(inner_env.get('port', '6379')),
            "password": inner_env.get('password'),
            "max_connections": max_connections,
            "decode_responses": inner_env.get('decode_responses', True)
        }
        return con_info


def get_db_key_list(
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name,
        auto_reconnect: bool = True,  # 自动重连
        reconnect_delay: int = 1  # 重连延时，单位为秒
):
    """
    获取 当前数据库的所有键列表
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.get_db_key_list(
        auto_reconnect=auto_reconnect,
        reconnect_delay=reconnect_delay
    )


def delete_key(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name,
        auto_reconnect: bool = True,  # 自动重连
        reconnect_delay: int = 1  # 重连延时，单位为秒
):
    """
    删除 key
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.delete_key(
        key=key,
        auto_reconnect=auto_reconnect,
        reconnect_delay=reconnect_delay
    )


def list_add_r(
        key,
        value,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.list_add_r(
        key=key,
        value=value
    )


def list_pop_l(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.list_pop_l(
        key=key
    )


def list_read_key_values(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.list_read_key_values(
        key=key
    )


def list_read_first_value(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # 获取列表的第一个元素
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.list_read_first_value(
        key=key
    )


def list_read_last_value(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # 获取列表的最后一个元素
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.list_read_last_value(
        key=key
    )


def count_set(
        key,
        value,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # 键 计数 设定值
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.set_save(
        name=key,
        value=value
    )


def count_add(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # 键 计数 增加1
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.count_add(
        name=key
    )


def count_reduce(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # 键 计数 减少1
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.count_reduce(
        name=key
    )


def count_read(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # 键 计数 获取值
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.count_read(
        key=key
    )


def list_get_values_count(
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    # 键 获取列表元素数量
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.list_get_values_count(
        key=key
    )


# -------------------- hash --------------------


def hash_set(
        name,
        key,
        value,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    """
    设置hash值

    :param name: 数据库层的键
    :param key: Hash的键
    :param value: Hash的值
    :param db: 数据库序号
    :param con_info: 连接信息
    :param env_file_name: 环境文件名
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.hash_set(
        name=name,
        key=key,
        value=value,
    )


def hash_delete(
        name,
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    """
    删除hash的一个键

    :param name: 数据库层的键
    :param key: Hash的键
    :param db: 数据库序号
    :param con_info: 连接信息
    :param env_file_name: 环境文件名
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.hash_delete(
        name=name,
        key=key
    )


def hash_keys(
        name,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    """
    获取hash所有键

    :param name: 数据库层的键
    :param db: 数据库序号
    :param con_info: 连接信息
    :param env_file_name: 环境文件名
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.hash_keys(
        name=name
    )


def hash_get(
        name,
        key,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    """
    获取hash的某个键的值，一次只获取一个键的值

    :param name: 数据库层的键
    :param key: Hash的键
    :param db: 数据库序号
    :param con_info: 连接信息
    :param env_file_name: 环境文件名
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.hash_get(
        name=name,
        key=key
    )


def hash_get_many(
        name,
        key_list: list,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name
):
    """
    获取hash的某个键的值，一次只获取多个键的值

    :param name: 数据库层的键
    :param key_list: Hash的键列表
    :param db: 数据库序号
    :param con_info: 连接信息
    :param env_file_name: 环境文件名
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.hash_get_many(
        name=name,
        key_list=key_list
    )


def hash_get_all(
        name,
        db: int = default_db,
        con_info: dict = None,  # 若指定，将优先使用
        env_file_name: str = default_env_file_name,
        auto_reconnect: bool = True,  # 自动重连
) -> dict:
    """
    获取hash的所有键的值

    :param name: 数据库层的键
    :param db: 数据库序号
    :param con_info: 连接信息
    :param env_file_name: 环境文件名
    :param auto_reconnect: 是否自动重连，默认为True
    """
    # ---------------- 固定设置 ----------------
    if con_info is None:
        con_info = make_con_info(
            env_file_name=env_file_name
        )
    con_info['db'] = db
    # ---------------- 固定设置 ----------------
    basics = Basics(
        con_info=con_info
    )
    return basics.hash_get_all(
        name=name,
        auto_reconnect=auto_reconnect
    )



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import platform
import shutil
import sys
import os
current_user_path = os.path.abspath('.')
if platform.system() == 'Windows':
    path_separator = '\\'
else:
    path_separator = '/'


def make_path(
        path: str = None,
        overwrite: bool = False,  # 是否覆盖
        silence: bool = False
):
    """
    按照目录逐级创建
    """
    if path is None:
        return
    else:
        path_list = str(path).split(path_separator)
        path_temp = ""
        for each_path in path_list:
            path_temp = path_temp + each_path + path_separator
            if overwrite is False:
                if os.path.exists(path_temp) is True:
                    if silence is True:
                        pass
                    else:
                        print("==> 目录[ %s ]已存在..." % path_temp)
                else:
                    if silence is True:
                        pass
                    else:
                        print("==> 目录[ %s ]创建中..." % path_temp)
                    os.mkdir(path_temp)
                    if silence is True:
                        pass
                    else:
                        print("==> 目录[ %s ]创建完成..." % path_temp)
            else:
                if os.path.exists(path_temp) is True:
                    if silence is True:
                        pass
                    else:
                        print("==> 目录[ %s ]已存在，将覆盖..." % path_temp)
                    shutil.rmtree(path_temp)
                    os.mkdir(path_temp)
                else:
                    if silence is True:
                        pass
                    else:
                        print("==> 目录[ %s ]创建中..." % path_temp)
                    os.mkdir(path_temp)
                    if silence is True:
                        pass
                    else:
                        print("==> 目录[ %s ]创建完成..." % path_temp)
        return path


def make_data_path(
        file_name: str
):
    file_name = os.path.basename(file_name).replace(".py", "")
    path = "data%s%s" % (path_separator, file_name)
    if os.path.exists(path) is True:
        pass
    else:
        make_path("data")
        os.mkdir(path)
    return path


def project_path(
        project_name
):
    # 获取项目的根目录，需要输入项目名称
    cur_path = os.path.abspath(os.path.dirname(__file__))
    _project_path = cur_path[:cur_path.find("%s%s" % (project_name, path_separator)) + len("%s%s" % (project_name, path_separator))]
    return _project_path


def file_path():
    """
    os.path.dirname(os.path.abspath(__file__))  单纯的文件地址
    os.path.dirname(os.path.realpath(__file__))  可能会存在的文件指向的真实地址
    """
    return os.path.dirname(os.path.abspath(__file__))


def get_all_top_dir():
    """
    枚举当前文件所处的文件夹的所有明细路径
    """
    file_dir = sys.argv[0]
    dir_list = file_dir.split(path_separator)
    dir_list_length = len(dir_list)
    new_dir_list = list()
    for i in range(dir_list_length):
        dir_list.pop(dir_list_length - i - 1)
        if len(dir_list) > 0:
            temp_dir = path_separator.join(dir_list)
            if len(temp_dir) > 0:
                new_dir_list.append(temp_dir)
            else:
                continue
        else:
            continue
    return new_dir_list


def visit_dir(
        path
):
    total_size = 0
    file_num = 0
    dir_num = 0
    for lists in os.listdir(path):
        sub_path = os.path.join(path, lists)
        # print(sub_path)
        if os.path.isfile(sub_path):
            file_num = file_num+1
            # print(fileNum)# 统计文件数量
            total_size = total_size+os.path.getsize(sub_path)  # 文件总大小
        elif os.path.isdir(sub_path):
            dir_num = dir_num+1  # 统计文件夹数量
            visit_dir(sub_path)  # 递归遍历子文件夹
    return file_num


def file_list(
        file_dir=os.path.dirname(os.path.realpath(__file__))
):
    return os.listdir(file_dir)



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import requests
import json


def get_public_ip():
    """
    获取当前网络公网ip地址
    """
    request_url = "http://httpbin.org/ip"
    response = requests.get(url=request_url)
    origin_ip = json.loads(response.text).get("origin")
    return origin_ip



#!/usr/bin/env python3
# coding = utf8
from . import lazyCrypto
from . import lazytime
from . import lazyfile
from . import lazypath
from . import lazymd5
import subprocess
import threading
import requests
import showlog
import shutil
import queue
import m3u8
import time
import sys
import os
import re


timeout = 10
default_headers = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate",
    "Connection": "keep-alive",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:81.0) Gecko/20100101 Firefox/81.0"
}
path_separator = lazypath.path_separator


def m3u8_file_maker(
        m3u8_url
):
    """
    输入m3u8的地址，生成m3u8文件，以供下载，当前不含解密部分
    生成的m3u8文件为可以离线直接下载或播放的文件
    主要是为了解决一些m3u8文件内容地址不完整的情况
    """
    m3u8_obj = m3u8.load(
        uri=m3u8_url,
        headers=default_headers,
        timeout=timeout,
        verify_ssl=False
    )  # 获取内容
    # ---------- 头部处理 ----------
    m3u8_obj_base_uri = m3u8_obj.base_uri
    # print('m3u8_obj_base_uri: %s' % m3u8_obj_base_uri)
    m3u8_base_url_head = re.findall('(.*?//)', m3u8_obj.base_uri, re.S)[0]  # http或https的头
    m3u8_base_url_heart = re.findall('//(.*?)/', m3u8_obj.base_uri, re.S)[0]  # 根域名内容
    m3u8_base_url = m3u8_base_url_head + m3u8_base_url_heart  # 拼接根域名
    # print('m3u8_base_url: %s' % m3u8_base_url)
    m3u8_base_url_body = m3u8_obj_base_uri.replace(m3u8_base_url, '')  # 根域名后面的内容（肚子1）
    # print('m3u8_base_url_body: %s' % m3u8_base_url_body)
    # ---------- 头部处理 ----------

    if m3u8_obj.is_variant is True:
        print('含有码流选择')
        for playlist in m3u8_obj.playlists:
            print(playlist.uri)
            print(playlist.stream_info.bandwidth)
        print('默认选择第1个')
        playlist_0 = m3u8_obj.playlists[0]
        url_new = m3u8_obj.base_uri + playlist_0.uri.replace(m3u8_base_url_body, '')  # 去除相同内容
        print('url_new: %s' % url_new)
        # ---------- 2次跳转处理 ----------
        url_new_body = url_new.replace(m3u8_obj_base_uri, '').replace('index.m3u8', '')  # 根域名后面的内容（肚子2）
        print(url_new_body)
        # ---------- 2次跳转处理 ----------

        m3u8_obj_p0 = m3u8.load(url_new, headers=default_headers, timeout=timeout, verify_ssl=False)

        m3u8_obj_p0_base_uri = m3u8_obj_p0.base_uri
        # print(m3u8_obj_p0_base_uri)  # 根uri
        m3u8_obj_p0_content = m3u8_obj_p0.dumps()
        m3u8_obj_p0_content = m3u8_obj_p0_content.split('\n')
        # print(m3u8_obj_p0_content)
        m3u8_obj_p0_content_pro = list()
        for line in range(len(m3u8_obj_p0_content)):
            each_line = m3u8_obj_p0_content[line]
            if '#' in each_line:
                # 标记行
                m3u8_obj_p0_content_pro.append(each_line)
                if '#EXTINF' in each_line:
                    data_url = m3u8_obj_p0_base_uri + m3u8_obj_p0_content[line + 1].replace(m3u8_base_url_body, '').replace(url_new_body, '')  # 去除相同内容
                    m3u8_obj_p0_content_pro.append(data_url)
            else:
                # 非标记行
                pass
        m3u8_obj_p0_content_pro_m3u8 = '\n'.join(m3u8_obj_p0_content_pro)
        return m3u8_obj_p0_content_pro_m3u8
    else:
        print('不含码流选择')
        # 检查链接是否完整
        m3u8_obj_content = m3u8_obj.dumps()
        m3u8_obj_content_split = m3u8_obj_content.split('\n')
        m3u8_line_list = list()
        for index, line in enumerate(m3u8_obj_content_split):
            if '#' in line:
                # 含有#的行为标记行
                m3u8_line_list.append(line)
                if '#EXTINF' in line:
                    if 'http' in m3u8_obj_content_split[index + 1]:
                        data_url = m3u8_obj_content_split[index + 1]
                    elif m3u8_base_url in m3u8_obj_content_split[index + 1]:
                        data_url = m3u8_obj_content_split[index + 1].replace(m3u8_base_url_body, '')  # 去除相同内容
                    else:
                        if m3u8_base_url_body in m3u8_base_url:
                            data_url = m3u8_base_url + m3u8_obj_content_split[index + 1].replace(m3u8_base_url_body,
                                                                                                 '')  # 去除相同内容
                        else:
                            data_url = m3u8_base_url + m3u8_base_url_body + m3u8_obj_content_split[index + 1].replace(
                                m3u8_base_url_body, '')  # 去除相同内容
                    m3u8_line_list.append(data_url)
            else:
                # 非标记行
                pass
        m3u8_file = '\n'.join(m3u8_line_list)
        return m3u8_file


def m3u8_checker(
        url
):
    """
    url为m3u8地址
    """
    m3u8_obj = m3u8.load(
        uri=url,
        headers=default_headers,
        timeout=timeout,
        verify_ssl=False
    )  # 获取内容
    # ---------- 头部处理 ----------
    m3u8_obj_base_uri = m3u8_obj.base_uri
    print('m3u8_obj_base_uri: %s' % m3u8_obj_base_uri)
    m3u8_base_url_head = re.findall('(.*?//)', m3u8_obj.base_uri, re.S)[0]  # http或https的头
    m3u8_base_url_heart = re.findall('//(.*?)/', m3u8_obj.base_uri, re.S)[0]  # 根域名内容
    m3u8_base_url = m3u8_base_url_head + m3u8_base_url_heart  # 拼接根域名
    print('m3u8_base_url: %s' % m3u8_base_url)
    m3u8_base_url_body = m3u8_obj_base_uri.replace(m3u8_base_url, '')  # 根域名后面的内容（肚子1）
    print('m3u8_base_url_body: %s' % m3u8_base_url_body)
    # ---------- 头部处理 ----------

    if m3u8_obj.is_variant is True:
        print('含有码流选择')
        for playlist in m3u8_obj.playlists:
            print(playlist.uri)
            print(playlist.stream_info.bandwidth)
        print('默认选择第1个')
        playlist_0 = m3u8_obj.playlists[0]
        url_new = m3u8_obj.base_uri + playlist_0.uri.replace(m3u8_base_url_body, '')  # 去除相同内容
        print('url_new: %s' % url_new)
        # ---------- 2次跳转处理 ----------
        url_new_body = url_new.replace(m3u8_obj_base_uri, '').replace('index.m3u8', '')  # 根域名后面的内容（肚子2）
        print(url_new_body)
        # ---------- 2次跳转处理 ----------

        m3u8_obj_p0 = m3u8.load(
            uri=url_new,
            headers=default_headers,
            timeout=timeout,
            verify_ssl=False
        )

        m3u8_obj_p0_base_uri = m3u8_obj_p0.base_uri
        print(m3u8_obj_p0_base_uri)  # 根uri
        m3u8_obj_p0_content = m3u8_obj_p0.dumps()
        m3u8_obj_p0_content = m3u8_obj_p0_content.split('\n')
        # print(m3u8_obj_p0_content)
        m3u8_obj_p0_content_pro = list()
        for line in range(len(m3u8_obj_p0_content)):
            each_line = m3u8_obj_p0_content[line]
            if '#' in each_line:
                # 标记行
                m3u8_obj_p0_content_pro.append(each_line)
                if '#EXTINF' in each_line:
                    data_url = m3u8_obj_p0_base_uri + m3u8_obj_p0_content[line + 1].replace(m3u8_base_url_body, '').replace(url_new_body, '')  # 去除相同内容
                    m3u8_obj_p0_content_pro.append(data_url)
            else:
                # 非标记行
                pass
        m3u8_obj_p0_content_pro_m3u8 = '\n'.join(m3u8_obj_p0_content_pro)
        return m3u8_obj_p0_content_pro_m3u8
    else:
        print('不含码流选择')
        return m3u8_obj.dumps()


def download_st_2(
        url_list,
        filename,
        aes_key=None,
        suffix_name="st",
        headers=None,
        save_path="download",
        temp_path='temp'
):
    """
    注意这里分段解码后拼接会出现视频花屏问题，所以不能分段，此模块不适用于需要解码的
    实现文件下载功能，可指定url、文件名、后缀名、请求头、文件保存路径
    :param url_list:
    :param filename:文件名
    :param aes_key: aes解密key
    :param suffix_name:后缀名
    :param headers:请求头
    :param save_path:文件保存路径
    :param temp_path:临时文件路径
    :return:
    """
    if headers is None:
        headers_local = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:68.0) Gecko/20100101 Firefox/68.0"}
    else:
        headers_local = headers

    if save_path is None:
        path_local = str(filename) + "." + str(suffix_name)
    else:
        if not os.path.exists(save_path):
            try:
                os.makedirs(save_path)
            except:
                pass
        path_local = save_path + '/' + str(filename) + "." + str(suffix_name)

    with open(path_local, "ab+") as f:
        url_count = 0
        for each_url in url_list:
            url_count += 1
            showlog.info("正在下载第 [ %s/%s ] 部分..." % (url_count, len(url_list)))
            while True:
                try:
                    print("下载进度：", end="")
                    time_pre = time.time()
                    response = requests.get(
                        url=each_url,
                        headers=headers_local,
                        stream=True
                    )
                    total_length = response.headers.get('content-length')

                    if total_length is None:  # no content length header
                        print("文件为空!")
                    else:
                        dl = 0
                        total_length = int(total_length)
                        char_list = ['\\', '|', '/', '-']
                        index = 0
                        for data in response.iter_content(chunk_size=8192):
                            dl += len(data)
                            if aes_key is None:
                                data_decode = data
                            else:
                                data_decode = lazyCrypto.aes_decode(
                                    data=data,
                                    key=aes_key
                                )
                            f.write(data_decode)
                            f.flush()
                            done = int(40 * dl / total_length)

                            time_now = time.time()
                            sys.stdout.write("\r下载中: %s [%s%s] %.2f%%  %.2fMB / %.2fMB SPEED: %.2fMB/s" % (
                            char_list[index], '=' * done, ' ' * (40 - done), (dl / total_length) * 100, dl / 1000000,
                            total_length / 1000000, (dl / (1024.0 * 1024.0 * 1024.0)) / ((time_now - time_pre) / 1000)))
                            sys.stdout.flush()
                            index = (index + 1) % len(char_list)
                    print("")
                    break
                except:
                    pass
    showlog.info("  ==> 保存位置:", path_local)


def worker(
        each_url,
        q,
        threading_num,
        aes_key=None,
        path_local='temp2.ts'
):
    path_local = 'temp/%s.ts' % threading_num
    for url in each_url:
        while True:
            try:
                response = requests.get(url=url, headers=default_headers, stream=True)
                break
            except:
                time.sleep(1)
        total_length = response.headers.get('content-length')

        if total_length is None:  # no content length header
            print("文件为空!")
        else:
            dl = 0
            total_length = int(total_length)
            char_list = ['\\', '|', '/', '-']

            with open(path_local, "ab+") as f:
                for data in response.iter_content(chunk_size=8192):
                    dl += len(data)
                    if aes_key is None:
                        data_decode = data
                    else:
                        data_decode = lazyCrypto.aes_decode(data, aes_key)
                    f.write(data_decode)
                    f.flush()


def multi_threading(
        task_list,
        each_thread_task_limit=20,
        thread_num=None
):
    """
    此模块是一个多线程控制模块
    task_list:任务列表
    each_thread_task_limit:每个线程的最大任务限制
    """
    showlog.info('本次任务总量：%s' % len(task_list))
    showlog.info('正在分配线程...')
    threads = []
    q = queue.Queue()
    threading_num = 0

    if len(task_list) <= each_thread_task_limit:
        # 任务量小于单个线程的最大任务量限制，设置为单线程
        showlog.info("任务量小于单线程最大任务量限制，将执行单线程模式")
        t = threading.Thread(
            target=worker,
            args=(task_list, q, threading_num)
        )
        # t.setDaemon(True)
        t.start()
        threads.append(t)
        threading_num += 1
        for thread in threads:
            thread.join(timeout=60 * 30)
    else:
        # 任务量大于单个线程的最大任务量限制，设置为多线程
        showlog.info("任务量大于单线程最大任务量限制，将执行多线程模式")
        task_list_temp = list()
        count_num = 0
        for each_task in task_list:
            task_list_temp.append(each_task)
            count_num += 1
            if count_num % each_thread_task_limit == 0 or count_num == len(task_list):
                t = threading.Thread(
                    target=worker,
                    args=(task_list_temp, q, threading_num)
                )
                # t.setDaemon(True)
                t.start()
                threads.append(t)
                threading_num += 1
                task_list_temp = list()
            else:
                continue
        for thread in threads:
            thread.join(timeout=60 * 30 / threading_num)
    showlog.info('线程分配完成，线程总计: %s' % len(threads))
    return


def fragment_merger(
        fragment_path,
        fragment_suffix='ts',
        fragment_keep=False,
        merge_file_name='merge',
        merge_file_path=None,
        merge_file_suffix='ts'
):
    """
    模块功能：将多个碎片文件合成一个大文件，将自动按照ts文件名的数字顺序排序依次合成
    注意：碎片文件名需按照数字顺序组织，否则将会出错
    :param fragment_path: [必填]碎片文件夹
    :param fragment_suffix: 碎片文件后缀名，缺省为ts
    :param fragment_keep: 碎片文件是否保留，默认不保留
    :param merge_file_name: 合成文件名，缺省为merge
    :param merge_file_path: 合成文件夹，缺省为当前文件夹
    :param merge_file_suffix: 合成文件后缀名，缺省为ts
    :return:
    """
    if merge_file_path is None:
        merge_file_dir_name = '%s.%s' % (merge_file_name, merge_file_suffix)
    else:
        # pathx.make_path(merge_file_path)
        lazypath.make_path(merge_file_path)
        merge_file_dir_name = '%s%s%s.%s' % (merge_file_path, path_separator, merge_file_name, merge_file_suffix)

    print(':) 正在合成碎片...')
    # fragment_list = filebox.file_read.dir_file_list(file_dir=fragment_path)  # 获取文件夹下的碎片文件列表
    fragment_list = lazyfile.dir_file_list(file_dir=fragment_path)  # 获取文件夹下的碎片文件列表
    fragment_index_list = list()
    for fragment in fragment_list:
        # 只对指定后缀的文件合成
        if fragment_suffix in fragment:
            fragment_index = fragment.replace('.%s' % fragment_suffix, '')  # 去掉文件的后缀名，以取文件名
            fragment_index_list.append(int(fragment_index))  # 将文件名转换为数字
        else:
            continue
    fragment_index_list.sort(reverse=False)  # 对文件名升序重排

    with open(merge_file_dir_name, "ab+") as f:
        for fragment_index_sort in fragment_index_list:
            fragment_sort = '%s%s%s.%s' % (fragment_path, path_separator, fragment_index_sort, fragment_suffix)
            x = open(fragment_sort, "rb")
            f.write(x.read())
            f.flush()
            x.close()
    print(':) 碎片合成完成')
    time.sleep(1)
    if fragment_keep is False:
        shutil.rmtree(fragment_path)  # 删除文件夹
    else:
        pass
    return merge_file_dir_name


def download_fragment(
        url_list,
        aes_key=None,
        headers=None,
        video_save_path=None,
        fragment_path=None,
        fragment_suffix='ts'
):
    """
    模块功能：下载碎片到指定文件夹
    :param url_list: 碎片下载地址列表
    :param aes_key: aes解密key
    :param headers:请求头
    :param video_save_path:文件保存路径
    :param fragment_path:碎片保存路径
    :param fragment_suffix:碎片文件后缀名
    :return:
    """
    if headers is None:
        headers_local = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36"}
    else:
        headers_local = headers

    if fragment_path is None:
        fragment_path = 'fragment_%s' % str(time.time()).replace('.', '')  # 使用时间戳命名
    else:
        pass

    if video_save_path is not None:
        fragment_path_new = '%s%s%s' % (video_save_path, path_separator, fragment_path)
    else:
        fragment_path_new = fragment_path
    lazypath.make_path(fragment_path_new)
    fragment_file_name_list = list()
    print('开始下载...')
    for index, each_url in enumerate(url_list):
        print(each_url)
        fragment_file_name = '%s%s%s.%s' % (fragment_path_new, path_separator, index, fragment_suffix)  # 生成碎片文件名
        # 开始下载碎片文件
        while True:
            try:
                with open(fragment_file_name, "ab+") as f:
                    time_pre = time.time()
                    response = requests.get(url=each_url,
                                            headers=headers_local,
                                            stream=True,
                                            timeout=10)
                    total_length = response.headers.get('content-length')

                    if total_length is None:
                        print("Attention: There is no content length in header!")  # 未返回长度信息
                    else:
                        dl = 0
                        total_length = int(total_length)
                        char_list = ['\\', '|', '/', '-']
                        _index_ = 0
                        for data in response.iter_content(chunk_size=8192):
                            dl += len(data)
                            if aes_key is None:
                                data_decode = data
                            else:
                                data_decode = lazyCrypto.aes_decode(data, aes_key)
                            f.write(data_decode)
                            f.flush()
                            done = int(40 * dl / total_length)
                            time_now = time.time()
                            sys.stdout.write("\r第 [ %s/%s ] 部分下载中: %s [%s%s] %.2f%%  %.2fMB / %.2fMB SPEED: %.2fMB/s" % (
                            index + 1, len(url_list), char_list[_index_], '=' * done, ' ' * (40 - done), (dl / total_length) * 100,
                            dl / 1000000, total_length / 1000000, (dl / (1024.0 * 1024.0 * 1024.0)) / ((time_now - time_pre) / 1000)))
                            sys.stdout.flush()
                            _index_ = (_index_ + 1) % len(char_list)
                    # print("")
                break
            except:
                print('下载超时，将在1秒后重试...')
                showlog.error('')
                os.remove(fragment_file_name)
                time.sleep(1)
                pass
        fragment_file_name_list.append(fragment_file_name)
        # 碎片文件下载完成

    print("\n:) 下载完成")
    download_res = {
        'fragment_path': fragment_path_new,
        'fragment_suffix': fragment_suffix,
        'fragment_file_name_list': fragment_file_name_list
    }
    return download_res


def download_m3u8(
        m3u8_link=None,
        m3u8_file_path=None,
        filename=lazytime.get_file_name(),
        url_v=None,
        video_suffix_name="ts",
        aes_key=None,
        proxies=None,
        video_save_path='download',
        overwrite=True,
        postfix="m3u8",
        fragment_path=None,
        fragment_suffix=None,
        fragment_merge=True,
        fragment_keep=False,
        headers=None
):
    """
    在同路径下载同名st文件
    :param m3u8_link:
    :param m3u8_file_path:
    :param filename:
    :param url_v: url前缀
    :param video_suffix_name:
    :param aes_key:
    :param proxies:
    :param video_save_path:
    :param overwrite: 覆盖现有文件
    :param postfix: 覆盖现有文件
    :param fragment: 是否下载碎片
    :param fragment_path: 碎片下载地址
    :param fragment_suffix: 碎片文件后缀
    :param fragment_merge: 是否合成碎片
    :return:
    """
    if m3u8_link is None and m3u8_file_path is None:
        print(':( 请传入有效的m3u8地址！')
        return False

    if m3u8_file_path is not None:
        print("正在解析m3u8文件...")
        all_content = lazyfile.read(
            file_name=m3u8_file_path,
            postfix=postfix
        )
        m3u8_lines = all_content.split("\n")  # 读取文件里的每一行
    else:
        return False

    # 通过判断文件头来确定是否是M3U8文件
    if m3u8_lines[0] != "#EXTM3U":
        print(":( 非M3U8文件")
        return False
    else:
        url_list = list()
        for index, line in enumerate(m3u8_lines):
            if "EXTINF" in line:
                url = m3u8_lines[index + 1]
                if url_v is None:
                    url_list.append(url)
                else:
                    url_list.append(url_v + url)
        print("解析到 [ %s ] 个下载地址，即将下载..." % len(url_list))
        download_res = download_fragment(
            url_list=url_list,
            video_save_path=video_save_path,
            aes_key=aes_key,
            fragment_path=fragment_path,
            fragment_suffix=fragment_suffix,
            headers=headers
        )
        if fragment_merge is True:
            print(':) 碎片下载完成，正在合成碎片...')
            merge_file_dir_name = fragment_merger(
                fragment_path=download_res.get('fragment_path'),
                fragment_suffix=download_res.get('fragment_suffix'),
                fragment_keep=fragment_keep,
                merge_file_name=filename,
                merge_file_path=video_save_path,
                merge_file_suffix=video_suffix_name
            )
            download_res['merge_file_dir_name'] = merge_file_dir_name
        else:
            print(':) 碎片下载完成')
        return download_res


def download_m3u8_to_file(
        m3u8_url=None,
        m3u8_file_dir=None,
        m3u8_content=None,
        m3u8_keep=False,
        video_name=None,
        video_save_path='download',
        video_suffix='ts',  # 视频文件设置
        overwrite=True,  # 是否覆盖原有视频文件
        fragment_path=None,
        fragment_suffix='ts',
        fragment_merge=True,
        fragment_keep=False
):
    """
    m3u8_url/m3u8_file_dir/m3u8_content 3选1，提供3种方式下载
    文件命名：video_name + video_biz_id + file_md5
    先下载碎片，再合成
    fragment_path:碎片保存地址
    fragment_merge:是否合成碎片
    fragment_keep: 是否保留碎片
    """
    temp_filename = str(time.time()).replace('.', '')  # 临时文件使用时间戳命名

    # -------------------- 获取m3u8内容 --------------------
    if m3u8_url is None:
        if m3u8_file_dir is None:
            if m3u8_content is None:
                return False
            else:
                lazyfile.save(
                    file_name=temp_filename,
                    content=m3u8_content,
                    postfix="m3u8",
                    path=video_save_path,
                    overwrite=overwrite
                )
                if video_save_path is None:
                    m3u8_file_dir_name = '%s.m3u8' % temp_filename
                else:
                    m3u8_file_path = r'%s%s%s' % (video_save_path, path_separator, temp_filename)
                    m3u8_file_dir_name = '%s.m3u8' % m3u8_file_path
        else:
            m3u8_file_dir_name = m3u8_file_dir  # 完整路径
    else:
        try:
            m3u8_file = m3u8_file_maker(m3u8_url=m3u8_url)
            print(':) 获取m3u8内容成功')
            lazyfile.save(
                file_name=temp_filename,
                content=m3u8_file,
                postfix="m3u8",
                path=video_save_path,
                overwrite=overwrite
            )
            if video_save_path is None:
                m3u8_file_dir_name = '%s.m3u8' % (temp_filename)
            else:
                m3u8_file_path = r'%s%s%s' % (video_save_path, path_separator, temp_filename)
                m3u8_file_dir_name = '%s.m3u8' % m3u8_file_path
        except:
            print(':( 获取m3u8内容失败')
            return False
    # -------------------- 获取m3u8内容 --------------------

    video_file_path = r'%s%s%s' % (video_save_path, path_separator, temp_filename)  # 视频文件路径
    video_file_dir_name = '%s.%s' % (video_file_path, video_suffix)

    print('开始下载m3u8碎片 [%s] ...' % temp_filename)
    start_time = time.time()
    download_res = download_m3u8(
        m3u8_file_path=m3u8_file_dir_name.replace('.m3u8', ''),
        filename=temp_filename,
        video_suffix_name=video_suffix,
        video_save_path=video_save_path,
        overwrite=overwrite,
        fragment_path=fragment_path,
        fragment_suffix=fragment_suffix,
        fragment_merge=fragment_merge,
        fragment_keep=fragment_keep
    )
    finish_time = time.time()
    download_duration = finish_time - start_time
    duration_res = lazytime.get_time_duration(int(download_duration))
    download_duration_str = duration_res.get('duration_str')
    if download_res is False:
        print(':( 下载失败')
        return False
    else:
        print(':) 下载完成，耗时：%s' % download_duration_str)
        merge_file_dir_name = download_res.get('merge_file_dir_name')
        file_md5 = lazymd5.md5_file(
            file_path=merge_file_dir_name
        )

        if m3u8_keep is False:
            os.remove(m3u8_file_dir_name)
        else:
            pass

        print(':) 将文件按照md5重命名')
        if video_name is None:
            new_name = file_md5
        else:
            new_name = '%s-%s' % (video_name, file_md5)

        file_path_md5 = '%s%s%s.%s' % (video_save_path, path_separator, new_name, video_suffix)

        if os.path.exists(file_path_md5):
            print('==> [warning] 文件 [%s] 已存在，将删除文件！' % file_path_md5)
            os.remove(file_path_md5)

        lazyfile.file_rename(
            before_name=video_file_dir_name,
            after_name=file_path_md5
        )  # 修改下载文件名称
        print(':) 重命名成功，新的文件名为：%s' % file_md5)

        file_size_dict = lazyfile.get_file_size(file_dir=file_path_md5)
        res_dict = {
            'file_path': file_path_md5,
            'file_md5': file_md5,
            'download_duration': download_duration_str,
            'file_size_str': file_size_dict.get('size_str'),
            'file_size_byte': file_size_dict.get('size_byte')
        }
        print(':) 最终下载得到的文件路径为：[ %s ]' % file_path_md5)
        print('=' * 50)
        return True, res_dict


def convert_video(
        video_input: str,
        video_output: str
) -> None:
    """
    Convert video from ts to mp4.
    可能会出现实用iMovie播放和苹果手机播放发生的掉帧问题，使用其他播放器未出现
    Parameters
    ----------
    video_input : str
        The input file name
    video_output : str
        The output file name
    """
    flags = ["ffmpeg", "-i", f"{video_input}.ts", "-acodec", "copy", "-vcodec", "copy", video_output]
    subprocess.Popen(flags, stdout=subprocess.DEVNULL).wait()
    # os.unlink(f"{video_input}.ts")  # 删除文件


if __name__ == '__main__':
    test_m3u8_url = input('请输入需要下载的m3u8地址：')
    download_m3u8_to_file(m3u8_url=test_m3u8_url)



import random
import uuid
import time

uuid_node = random.randint(100000000, 999999999)  # 避免泄密设备信息


def random_mac_uuid():
    """
    uuid1
    由MAC地址、当前时间戳、随机数生成。可以保证全球范围内的唯一性，
    但MAC的使用同时带来安全性问题，局域网中可以使用IP来代替MAC。
    这里使用随机数代替
    """
    return str(uuid.uuid1(node=uuid_node))  # 替换原来的MAC地址为随机数


def timestamp_uuid():
    """
    使用纳秒级别时间戳+uuid作为id
    """
    return f'{time.time_ns()}_{random_mac_uuid()}'



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import datetime
import pytz
import time
import copy
import sys


def get_utc_datetime(
        f: str = '%Y-%m-%dT%H:%M:%SZ'
):
    """
    获取utc时间
    """
    utc_datetime_str = datetime.datetime.utcnow().strftime(f)
    return utc_datetime_str


def get_utc_timestamp():
    """
    获取utc时间的时间戳
    """
    utc_timestamp_str = datetime.datetime.utcnow().timestamp()
    return utc_timestamp_str


def get_utc_time_dict(
        f: str = '%Y-%m-%dT%H:%M:%SZ'
):
    """
    获取utc时间的时间和时间戳字典
    """
    utc_time_now = datetime.datetime.utcnow()
    utc_datetime_str = utc_time_now.strftime(f)
    utc_timestamp_str = utc_time_now.timestamp()
    return {'utc_datetime_str': utc_datetime_str, 'utc_timestamp_str': utc_timestamp_str}


def get_year(
        date_input: str = None,  # 输入日期
        date_delimiter: str = "-"  # 日期分隔符
):
    """
    获取当前系统的当前时间：年份
    :return: 2018
    """
    if date_input is None:
        year = datetime.datetime.now().year
    else:
        a1 = date_input.find(date_delimiter, 0)
        year = int(date_input[0:a1])
    return year


def get_month(
        date_input: str = None,  # 输入日期
        date_delimiter: str = "-"  # 日期分隔符
):
    """
    获取当前系统的当前时间：月份
    :return: 8
    """
    if date_input is None:
        month = datetime.datetime.now().month
    else:
        date = str(date_input)
        a1 = date.find(date_delimiter, 0)
        a2 = date.find(date_delimiter, a1 + 1)
        month = int(date[a1 + 1:a2])
    return month


def get_day(
        date_input: str = None,  # 输入日期
        date_delimiter: str = "-"  # 日期分隔符
):
    """
    获取当前系统的当前时间：日数
    :return: 1
    """
    if date_input is None:
        day = datetime.datetime.now().day
    else:
        date = str(date_input)
        a1 = date.find(date_delimiter, 0)
        a2 = date.find(date_delimiter, a1 + 1)
        day = int(date[a2 + 1:len(date)])
    return day


def get_hour():
    """
    获取当前系统的当前时间：小时数（24小时制）
    :return: 14
    """
    hour = datetime.datetime.now().hour
    return hour


def get_minute():
    """
    获取当前系统的当前时间：分钟数
    :return: 28
    """
    minute = datetime.datetime.now().minute
    return minute


def get_second():
    """
    获取当前系统的当前时间：秒数
    :return: 23
    """
    second = datetime.datetime.now().second
    return second


def get_time(
        f: str = '%H:%M:%S'
):
    """
    获取当前系统的当前时间格式的时间，精确到秒且只有时间
    :return: 14:20:41
    """
    inner_now = datetime.datetime.now()
    data_time = inner_now.strftime(f)
    return data_time


def get_datetime(
        f: str = '%Y-%m-%d %H:%M:%S'
):
    """
    获取当前系统的当前时间格式的时间，精确到秒
    :return: 2018-08-01 14:18:31
    """
    inner_now = datetime.datetime.now()
    data_time = inner_now.strftime(f)
    return data_time


def get_datetime_full():
    """
    获取当前系统的当前时间格式的时间，精确度最高
    :return: 2018-08-01 14:16:50.611705
    """
    inner_now = datetime.datetime.now()
    return inner_now


def get_datetime_str_int(
        f: str = '%Y%m%d%H%M%S'
):
    """
    获取当前系统的当前时间格式的时间，精确到秒
    :return: 20180801141831
    """
    inner_now = datetime.datetime.now()
    data_time = inner_now.strftime(f)
    return data_time


def get_relative_date(
        num: int = 0
):
    """
    获取当前系统当前时间的相对时间：日期，num为0表示当日，num为负数表示向历史推算的天数，num为正数表示向未来推算的天数
    :param num:
    :return: 2018-08-01
    """
    today = datetime.date.today()
    date = today - datetime.timedelta(days=-num)
    return date


def get_relative_datetime(
        num: int = 0,
        f: str = '%Y-%m-%d %H:%M:%S'
):
    """
    【计划弃用】
    获取当前系统当前时间的相对时间：日期，num为0表示当日，num为负数表示向历史推算的天数，num为正数表示向未来推算的天数
    :param num:
    :param f: 格式
    :return: 2021-04-23 17:23:27
    """
    today = datetime.datetime.now()
    date = today - datetime.timedelta(days=-num)
    return date.strftime(f)


def get_datetime_relative(
        days: int = 0,
        f: str = '%Y-%m-%d %H:%M:%S'
) -> str:
    """
    获取当前系统当前时间的相对时间

    :param days: 0表示当日，num为负数表示向历史推算的天数，num为正数表示向未来推算的天数
    :param f: 格式
    :return: 形如 2022-06-07 12:16:52 的字符串
    """
    target_date = datetime.datetime.now() - datetime.timedelta(days=-days)
    return target_date.strftime(f)


def get_datetime_relative_min(
        days: int = 0,
        f: str = '%Y-%m-%d %H:%M:%S'
) -> str:
    """
    获取当前系统当前时间的相对时间 的一天的 开始时间，例如：2022-06-07 00:00:00

    :param days: 0表示当日，num为负数表示向历史推算的天数，num为正数表示向未来推算的天数
    :param f: 格式
    :return: 形如 2022-06-07 00:00:00 的字符串
    """
    target_date = datetime.date.today() - datetime.timedelta(days=-days)
    return datetime.datetime.combine(
        date=target_date,
        time=datetime.time.min
    ).strftime(f)


def get_datetime_relative_max(
        days: int = 0,
        f: str = '%Y-%m-%d %H:%M:%S'
) -> str:
    """
    获取当前系统当前时间的相对时间 的一天的 结束时间，例如：2022-06-07 23:59:59

    :param days: 0表示当日，num为负数表示向历史推算的天数，num为正数表示向未来推算的天数
    :param f: 格式
    :return: 形如 2022-06-07 23:59:59 的字符串
    """
    target_date = datetime.date.today() - datetime.timedelta(days=-days)
    return datetime.datetime.combine(
        date=target_date,
        time=datetime.time.max
    ).strftime(f)


def get_timestamp():
    """
    获取当前系统的当前时间戳格式的时间，返回的类型为int
    :return: 1533104393
    """
    inner_now = time.time()
    return int(inner_now)


def get_timestamp2datetime(
        timestamp: int = None,
        f: str = "%Y-%m-%d %H:%M:%S",
        zero_to_none: bool = True
):
    """
    将时间戳转换为datetime时间
    :param timestamp: 同时支持字符串和数字格式
    :param f:
    :param zero_to_none: 将0转换为空值
    :return: 2018-08-01 14:19:53
    """
    if timestamp is not None:
        if timestamp == 0 and zero_to_none is True:
            return
        else:
            time_array = time.localtime(timestamp)
            date_time = time.strftime(f, time_array)
            return date_time
    else:
        return


def get_timestamp2date(
        timestamp: int = None,
        f: str = "%Y-%m-%d",
        zero_to_none: bool = True
):
    """
    将时间戳转换为datetime时间
    默认将输入转换为int
    :param timestamp:
    :param f:
    :param zero_to_none: 将0转换为空值
    :return: 2018-08-01 14:19:53
    """
    if timestamp is not None:
        if timestamp == 0 and zero_to_none is True:
            return
        else:
            time_array = time.localtime(timestamp)
            date_time = time.strftime(f, time_array)
            return date_time
    else:
        return


def get_date2timestamp(
        date: str,
        f: str = "%Y-%m-%d"
):
    """
    将日期转换为对应日期的0点的时间戳
    :param date:
    :param f:
    :return: 1533052800
    """
    time_array = time.strptime(date, f)
    timestamp = int(time.mktime(time_array))
    return timestamp


def get_datetime2timestamp(
        data: str,
        f: str = "%Y-%m-%d %H:%M:%S"
):
    time_array = time.strptime(data, f)
    timestamp = int(time.mktime(time_array))
    return timestamp


def get_datetime2date(
        datetime_str: str
):
    datetime_timestamp = get_datetime2timestamp(datetime_str)
    date_str = get_timestamp2date(datetime_timestamp)
    return date_str


def timestamp_day_num_start(
        num: int = 0
):
    """
    换算出相对时间的当日开始时间的时间戳
    :param num:
    :return: 1533052800（2018-08-01 00:00:00）
    """
    inner = get_relative_date(num=num)
    return get_date2timestamp(str(inner))


def timestamp_day_num_end(
        num: int = 0
):
    """
    换算出相对时间的当日结束时间的时间戳
    :param num:
    :return: 1533139199（2018-08-01 23:59:59）
    """
    inner = get_relative_date(num=num+1)
    return get_date2timestamp(str(inner))-1


def get_format_date(
        date_ori: str,
        date_delimiter: str = '-',  # 日期分隔符号
):
    """
    将以'-'连接的字符串的日期格式化为日期格式
    :param date_ori:
    :param date_delimiter:
    :return: 2018-01-01
    """
    a1 = date_ori.find(date_delimiter, 0)
    a2 = date_ori.find(date_delimiter, a1+1)
    year = int(date_ori[0:a1])
    month = int(date_ori[a1+1:a2])
    day = int(date_ori[a2+1:len(date_ori)])
    format_date = datetime.date(year, month, day)
    return format_date


def get_format_date_2(
        date_ori: str
):
    """
    将以的字符串的日期格式化为日期格式:20200602
    按照位置分
    :param date_ori:
    :return: 2018-01-01
    """
    year = int(date_ori[0:4])
    month = int(date_ori[4:6])
    day = int(date_ori[6:8])
    format_date = datetime.date(year, month, day)
    return format_date


def get_format_datetime(
        datetime_ori: str,
        date_delimiter: str = '-',  # 日期分隔符号
        space_delimiter: str = " ",  # 空格分隔符号
        time_delimiter: str = ':'  # 时间分隔符号
):
    """
    将以'-'连接的字符串的日期格式化为日期格式
    :param datetime_ori:
    :param date_delimiter: 日期分隔符号
    :param space_delimiter: 空格分隔符号
    :param time_delimiter: 时间分隔符号
    :return: 2018-01-01
    """
    date_str, time_str = datetime_ori.split(space_delimiter)

    date_str = str(date_str)
    d1 = date_str.find(date_delimiter, 0)
    d2 = date_str.find(date_delimiter, d1+1)
    year_num = int(date_str[0:d1])
    month_num = int(date_str[d1+1:d2])
    day_num = int(date_str[d2+1:len(date_str)])

    time_str = str(time_str)
    t1 = time_str.find(time_delimiter, 0)
    t2 = time_str.find(time_delimiter, t1 + 1)
    hour_num = int(time_str[0:t1])
    minute_num = int(time_str[t1 + 1:t2])
    second_num = int(time_str[t2 + 1:len(time_str)])

    format_date = datetime.datetime(year_num, month_num, day_num, hour_num, minute_num, second_num)
    return format_date


def time_gap_seconds(
        start_time: str,
        end_time: str
):
    """
    计算两个时间的间隔秒数
    :param start_time:
    :param end_time:
    :return:
    """
    start_time_f = get_format_datetime(start_time)
    end_time_f = get_format_datetime(end_time)
    return (end_time_f - start_time_f).seconds


def time_gap_days(
        start_time: str,
        end_time: str
):
    """
    计算两个时间的间隔天数
    :param start_time:
    :param end_time:
    :return:
    """
    start_time_f = get_format_date(start_time)
    end_time_f = get_format_date(end_time)
    return (end_time_f - start_time_f).days


def get_add_date(
        date_input,
        num: int = 0,
        f: str = '%Y-%m-%d'
):
    """
    计算指定日期（date_ori）的相对日期
    :param date_input: 输入的指定日期，类型支持str、datetime.date，例如：2022-01-01
    :param num: 偏移天数，-1表示向过去偏移一天，1表示向未来偏移1天，0表示当天，默认值为0
    :param f: 输出时间格式
    :return: 2018-01-02
    """
    if isinstance(date_input, str):
        date = get_format_date(date_input)
    elif isinstance(date_input, datetime.date):
        date = date_input
    else:
        return
    delta = datetime.timedelta(days=num)
    n_days = date + delta
    date_add = n_days.strftime(f)
    return date_add


def get_timestamp_interval_seconds(
        timestamp: int
):
    """
    计算指定时间戳距离当前系统的当前时间的秒数，正数表示未来时间，负数表示过去时间
    :param timestamp:
    :return: 30830
    """
    inner_now = time.time()
    res = timestamp - int(inner_now)
    return res


def count_down(
        num: int
):
    """
    倒数计时器，按照指定的秒数原地倒计时刷新数字
    :param num:
    :return:
    """
    count = 0
    while count < num:
        n_count = num - count
        sys.stdout.write("\r%d " % n_count)
        sys.stdout.flush()
        time.sleep(1)
        count += 1


def running_controller(
        start_running_time: str,
        end_running_time: str,
        start_running_time_f: str = '%H:%M:%S',
        end_running_time_f: str = '%H:%M:%S'
):
    """
    判断系统时间是否落在设定的时间区间内，是则输出True，否则输出False
    :param start_running_time:
    :param end_running_time:
    :param start_running_time_f:
    :param end_running_time_f:
    :return:
    """
    if (start_running_time is None) and (end_running_time is None):
        return True
    else:
        inner_now = datetime.datetime.strptime(str(get_time()), "%H:%M:%S")
        inner_start = datetime.datetime.strptime(start_running_time, start_running_time_f)
        inner_end = datetime.datetime.strptime(end_running_time, end_running_time_f)

        if (inner_now >= inner_start) and (inner_now < inner_end):
            return True
        else:
            return False


def now():
    inner_now = datetime.datetime.now()
    return inner_now


def date_string(
        date=now(),
        f: str = '%Y-%m-%d'
):
    """
    将输入的时间转换为日期格式的字符串，如果不传入参数将取当前系统时间
    :param date:
    :return:
    """
    date_string_in = date.strftime(f)
    return date_string_in


def time_string(
        date=now(),
        f: str = '%Y-%m-%d %H-%M-%S'
):
    time_string_in = date.strftime(f)
    return time_string_in


def datetime_string(
        date=now(),
        f: str = '%Y-%m-%d %H-%M-%S'
):
    time_string_in = date.strftime(f)
    return time_string_in


def datetime_string_chs(
        date=now(),
        f: str = '%Y年%m月%d日%H时%M分%S秒'
):
    time_string_in = date.strftime(f)
    return time_string_in


def date_str_list(
        start_date: str,
        end_date: str
):
    # 生成起止时间之间的时间序列
    start_date_f = get_format_date(start_date)
    end_date_f = get_format_date(end_date)
    date_list = list()
    date_list.append(str(start_date_f))
    added_date = start_date_f
    while True:
        added_date = get_add_date(added_date, 1)
        added_date_f = get_format_date(added_date)
        if added_date_f > end_date_f:
            break
        else:
            date_list.append(str(added_date))
    return date_list


def date_str_list_form_now(
        day_num: int = 1
):
    start_date = get_add_date(date_string(), -day_num)
    end_date = date_string()
    res_list = date_str_list(
        start_date=start_date,
        end_date=end_date
    )
    return res_list


def get_normalized_date_string(
        days: int = 0,
        f: str = '%Y-%m-%d %H:%M:%S'
):
    """
    获取多少天以前的时间字符串
    :param days: 多少天以前
    :return: 时间字符串(xxxx-xx-xx xx:xx:xx)
    """
    current_time = datetime.datetime.now()
    target_time = current_time - datetime.timedelta(days=days)
    normalized_target_time = target_time.strftime(f)
    return normalized_target_time


def get_data_date_string(
        days: int = 0,
        f: str = '%Y-%m-%d'
):
    """
    获取多少天以前的时间字符串
    :param days: 多少天以前
    :return: 时间字符串(xxxx-xx-xx)
    """
    current_time = datetime.datetime.now()
    target_time = current_time - datetime.timedelta(days=days)
    date_time = target_time.strftime(f)
    return date_time


def get_date_string(
        days: int = 0,
        f: str = '%Y-%m-%d'
):
    """
    获取多少天以后的时间字符串
    :param days: 多少天以后，正数向未来计算，负数向历史计算，0是当天
    :return: 时间字符串(xxxx-xx-xx)
    """
    current_time = datetime.datetime.now()
    target_time = current_time + datetime.timedelta(days=days)
    date_str = target_time.strftime(f)
    return date_str


def time_day_num_start(
        num: int = 0
):
    """
    换算出相对时间的当日开始时间的时间
    :param num:
    :return: 2018-08-01 00:00:00
    """
    now = datetime.datetime.now()
    return now - datetime.timedelta(
        days=num,
        hours=now.hour,
        minutes=now.minute,
        seconds=now.second,
        microseconds=now.microsecond
    )


def time_day_num_end(
        num: int = 0
):
    """
    换算出相对时间的当日结束时间的时间
    :param num:
    :return: 2018-08-01 23:59:59
    """
    now = datetime.datetime.now()
    return now - datetime.timedelta(
        days=num - 1,
        hours=now.hour,
        minutes=now.minute,
        seconds=now.second + 1,
        microseconds=now.microsecond
    )


def timestamp_list_splitter(
        timestamp_list: list,
        n: int = 2
):
    # 时间戳范围拆分器
    """
    输入timestamp_list：[[start,end]]，splitter_num：2
    执行将list中的每段时间按照设定的份数拆分，拆分为粗略拆分，

    :return:
    """
    t_list_new = list()
    for each_t_list in timestamp_list:
        a = each_t_list[0]  # 获取开始时间戳
        b = each_t_list[1]  # 获取结束时间戳
        if (b - a) > 1:  # 当数字间隔大于1的时候才有拆分意义
            m = round((b - a)/n, 0)  # 计算加值数
            if m > 0:
                pass
            elif m == 0:
                m = 1
            else:
                continue
            t_list_new_temp = [[a, int(a + m)], [int(a + m + 1), b]]
            t_list_new.extend(t_list_new_temp)
        else:
            t_list_new.extend([each_t_list])
    return t_list_new


def date_gap_splitter(
        start_date: str,  # 开始日期
        end_date: str,  # 结束日期
        splitter_gap: int = 1,  # 拆分计算间隔
        successive: bool = False  # 结果是否连续
):
    """
    时间拆分器，将按照起止时间和间隔时间拆分时间段
    拆分从前向后拆分
    :return:
    """
    day_gap = time_gap_days(start_date, end_date)
    if splitter_gap >= day_gap:
        return [[start_date, end_date]]
    else:
        res_list = list()
        start_date_temp = start_date
        add_count = 0
        while True:
            end_date_temp = get_add_date(start_date_temp, splitter_gap)
            day_gap -= splitter_gap
            finish_num = get_date2timestamp(end_date) - get_date2timestamp(end_date_temp)
            if finish_num <= 0:
                res_list.append([str(start_date_temp), str(end_date)])
                break
            else:
                res_list.append([str(start_date_temp), str(end_date_temp)])
                if successive is True:
                    start_date_temp = copy.deepcopy(end_date_temp)
                else:
                    start_date_temp = get_add_date(end_date_temp, 1)
            add_count += 1
        return res_list


def get_time_duration(
        duration: int
):
    # 计算秒数的时长
    # start_duration = - 8 * 60 * 60  # 固定值
    temp_datetime = datetime.datetime(1970, 1, 1) + datetime.timedelta(seconds=duration)
    # temp_datetime = get_timestamp2datetime(duration + start_duration)  # 计算1970-01-01 00：00：00 后指定秒的日期
    temp_date, temp_time = str(temp_datetime).split(' ')  # 分割日期和时间
    temp_day = time_gap_days(start_time='1970-01-01', end_time=temp_date)  # 计算天数间隔
    if temp_day == 0:
        duration_str = temp_time
    else:
        duration_str = '%sd %s' % (temp_day, temp_time)
    res = {
        'duration_days': temp_day,
        'duration_time': temp_time,
        'duration_str': duration_str,
        'duration': duration
    }
    return res


def print_t(
        text
):
    print("%s >> %s" % ((datetime.datetime.now()), text))


def utc_format(
        utc_time=None,
        timezone_local="Asia/shanghai",
        input_format="%Y-%m-%dT%H:%M:%S.%fZ",
        output_format="%Y-%m-%d %H:%M:%S"
):
    if utc_time is None:
        utc_time = datetime.datetime.utcnow()
    else:
        pass
    local_tz = pytz.timezone(timezone_local)
    if isinstance(utc_time, datetime.datetime) is True:
        utc_datetime = utc_time
    else:
        utc_datetime = datetime.datetime.strptime(utc_time, input_format)
    utc_timestamp = utc_datetime.timestamp()
    utc_datetime_str = utc_datetime.strftime("%Y-%m-%dT%H:%M:%S.%fZ")

    local_datetime = utc_datetime.replace(tzinfo=pytz.utc).astimezone(local_tz)
    local_timestamp = local_datetime.timestamp()
    local_datetime_str = local_datetime.strftime(output_format)
    res = {
        'utc_timestamp': int(utc_timestamp),
        'utc_timestamp_m': int(utc_timestamp * 1000),
        'utc_datetime_str': utc_datetime_str,
        'local_timestamp': int(local_timestamp),
        'local_timestamp_m': int(local_timestamp * 1000),
        'local_datetime_str': local_datetime_str,
    }
    return res


def get_file_name(
        date_delimiter: str = '-',  # 日期分隔符号
        space_delimiter: str = "+",  # 空格分隔符号
        time_delimiter: str = '-'  # 时间分隔符号
):
    """
    获取当前系统的当前时间格式的时间，精确到秒
    :return: 2018年08月01日 14时18分31秒
    """
    inner_now = datetime.datetime.now()
    f_list = ['%Y', date_delimiter, '%m', date_delimiter, '%d', space_delimiter, '%H', time_delimiter, '%M', time_delimiter, '%S']
    f_str = ''.join(f_list)
    data_time = inner_now.strftime(f_str)
    return data_time


def format_seconds(
        input_seconds
):
    """
    输入秒数，以时间形式输出
    """
    seconds = int(input_seconds)
    microseconds = (input_seconds * 1000000) % 1000000  # 精确到微秒
    format_time = datetime.timedelta(
        days=0,
        seconds=seconds,  # 秒
        microseconds=microseconds  # 微秒
    )
    return format_time


def get_gap_days(
        date1: str,
        date2: str,
        data1_f: str = '%Y-%m-%d',
        data2_f: str = '%Y-%m-%d'
):
    """
    计算两个日期之间相差的天数
    """
    date1_date = datetime.datetime.strptime(date1, data1_f).date()
    date2_date = datetime.datetime.strptime(date2, data2_f).date()
    days = (date2_date - date1_date).days
    return days



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
from . import lazypath
import collections
import subprocess
import requests
import platform
import datetime
import openpyxl
import showlog
import time
import xlrd
import json
import sys
import os


if platform.system() == 'Windows':
    path_separator = '\\'
else:
    path_separator = '/'
headers_default = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:68.0) Gecko/20100101 Firefox/68.0"}


def get_suffix(
        file_name: str
):
    suffix = file_name.split('.')[-1]  # 获取文件后缀名
    return suffix


def file_rename(
        before_name: str,  # 原名称
        after_name: str  # 重命名名称
):
    """
    重命名文件，注意是完整文件路径
    :param before_name:
    :param after_name:
    :return:
    """
    try:
        os.rename(before_name, after_name)
        return True
    except Exception as e:
        print(e)
        print('rename file fail\r\n')
        return False


def open_folder(
        folder: str = os.path.dirname(os.path.abspath(__file__))  # 默认为当前路径
):
    """
    功能：打开路径
    """
    if platform.system() == 'Windows':
        os.startfile(folder)  # Windows上打开文件
    else:
        subprocess.check_call(['open', folder])  # 非Windows上打开文件


def download(
        url,
        filename: str = None,
        suffix_name: str = None,
        headers: dict = None,
        path: str = "download",
        proxies=None,
        size_limit: int = None,
        range_start: int = None,
        range_end: int = None,
):
    """
    实现文件下载功能，可指定url、文件名、后缀名、请求头、文件保存路径
    :param url:
    :param filename:文件名
    :param suffix_name:后缀名
    :param headers:请求头
    :param path:文件保存路径
    :param proxies:代理
    :param size_limit:尺寸限制
    :param range_start:开始位置
    :param range_end:结束位置
    :return:
    """
    if headers is None:
        headers_local = headers_default
    else:
        headers_local = headers

    if range_start is None and range_end is None:
        range_start = 0
        range_info = None
    elif range_start is not None and range_end is None:
        range_info = 'bytes=%d-' % range_start  # 从这里向后
    elif range_start is None and range_end is not None:
        range_start = 0
        range_info = 'bytes=0-%d' % range_end
    else:
        range_info = 'bytes=%d-%d' % (range_start, range_end)

    if range_info is None:
        pass
    else:
        headers_local['Range'] = range_info
    # 获取文件的基本信息
    response = requests.get(
        url=url,
        headers=headers_local,
        stream=True,
        proxies=proxies
    )
    total_length = response.headers.get('content-length')  # 文件大小
    content_type = response.headers.get('content-type')  # 文件类型
    content_disposition = response.headers.get('content-disposition')  # 文件名及类型
    filename_default = 'unknown_' + str(time.time())
    if content_disposition is not None:
        content_dispositions = content_disposition.replace(' ', '').split(';')
        for each_content_disposition in content_dispositions:
            if 'filename' in each_content_disposition:
                each_content_disposition_split = each_content_disposition.split(sep='=', maxsplit=1)  # 只拆分一次，防止有多个=影响
                filename_default_full = each_content_disposition_split[1]
                filename_default = filename_default_full[:filename_default_full.rfind('.')]  # 解析文件名
                suffix_name = filename_default_full[filename_default_full.rfind('.')+1:]  # 解析文件后缀
            else:
                pass
    else:
        pass

    if suffix_name is None:
        # 尝试自动获取文件后缀名
        suffix_name = content_type.split('/')[1]

    if filename is None:
        download_file_name = str(filename_default) + "." + str(suffix_name)
    else:
        download_file_name = str(filename) + "." + str(suffix_name)

    if path is None:
        path_local = download_file_name
    else:
        if os.path.exists(path):
            pass
        else:
            os.makedirs(path)
        path_local = path + path_separator + download_file_name

    if range_start is None:
        temp_size = 0  # 已经下载文件大小
    else:
        temp_size = range_start + 0  # 已经下载文件大小
    chunk_size = 1024  # 分割文件大小，字节B
    total_size = int(total_length)  # 文件总大小
    total_size_mb = round(total_size / (1024 * 1024), 2)  # 换算到MB的文件大小
    # 添加文件大小控制，跳过下载超大文件
    if size_limit is None:
        pass
    else:
        if total_size > size_limit:
            return
        else:
            pass

    time_start = time.time()  # 获取下载开始时间
    is_finish = False

    with open(path_local, "ab") as f:  # wb新建文件，a追加
        for chunk in response.iter_content(chunk_size=chunk_size):
            try:
                temp_time_now = time.time()  # 时间采样
                time_spend_total = temp_time_now - time_start
                if time_spend_total == 0:
                    total_speed = round((temp_size-range_start) / (1024 * 1024) / 0.001, 2)  # 计算速度：MB/s
                else:
                    total_speed = round((temp_size-range_start) / (1024 * 1024) / time_spend_total, 2)  # 计算速度：MB/s
                if not chunk:
                    if temp_size >= total_size:
                        is_finish = True
                    else:
                        is_finish = False
                    break
                else:
                    temp_size += len(chunk)
                    f.write(chunk)
                    f.flush()
                    done = int(50 * temp_size / total_size)
                    if total_speed == 0 or time_spend_total == 0:
                        time_left = 0
                    else:
                        time_left = (total_size - temp_size) / total_speed / 1024 / 1024
                    show_dict = {
                        'finish_mark': '█' * done,
                        'not_finish_mark': ' ' * (50 - done),
                        'total_size': total_size_mb,  # 换算到M
                        'total_percent': round(100 * temp_size / total_size, 4),
                        'total_speed': total_speed,
                        'finish_size': round(temp_size / (1024 * 1024), 2),
                        'time_spend_total': int(time_spend_total),
                        'time_left': int(time_left)
                    }
                    show_msg = "\r[%(finish_mark)s%(not_finish_mark)s] " \
                               "总大小:%(total_size)sMB " \
                               "总进度:%(total_percent)s%% " \
                               "平均速度:%(total_speed)sMB/s " \
                               "已下载:%(finish_size)sMB " \
                               "已耗时 %(time_spend_total)s 秒 " \
                               "预计剩余 %(time_left)s 秒" % show_dict
                    sys.stdout.write(show_msg)
                    sys.stdout.flush()
                    if temp_size >= total_size:
                        is_finish = True
                    else:
                        is_finish = False
            except:
                showlog.error('')
    print("\n  ==> 文件已全部下载完成，保存位置:", path_local)
    res_dict = {
        'file_dir': path_local,
        'is_finish': is_finish,
        'temp_size': temp_size
    }
    return res_dict


def safe_download(
        url,
        filename=None,
        suffix_name=None,
        headers=None,
        path="download",
        proxies=None,
        size_limit=None,
        range_start=None,
        range_end=None
):
    while True:
        download_response = download(
            url=url,
            filename=filename,
            suffix_name=suffix_name,
            headers=headers,
            path=path,
            proxies=proxies,
            size_limit=size_limit,
            range_start=range_start,
            range_end=range_end
        )
        if download_response.get('is_finish') is True:
            local_file_dir = download_response.get('file_dir')
            return local_file_dir
        else:
            print(':( 下载中断')
            range_start = download_response.get('temp_size')
            time.sleep(1)
            print('将继续下载（断点续传）...')


def read(
        file_name,
        postfix="txt",
        path=None,
        silence=False
):
    try:
        if postfix == "txt":
            if path is None:
                f = open("%s.%s" % (file_name, postfix), encoding='utf-8')
            else:
                f = open("%s/%s.%s" % (path, file_name, postfix), encoding='utf-8')
            res = f.read()
            return res
        elif postfix == "json":
            content_list = list()
            if path is None:
                with open("%s.%s" % (file_name, postfix), 'r', encoding='utf-8') as f:
                    content = f.readlines()
                    for each in content:
                        each_json = json.loads(each)
                        content_list.append(each_json)
            else:
                with open("./%s/%s.%s" % (path, file_name, postfix), 'r', encoding='utf-8') as f:
                    content = f.readlines()
                    for each in content:
                        each_json = json.loads(each)
                        content_list.append(each_json)
            return content_list
        else:
            if path is None:
                f = open("%s.%s" % (file_name, postfix), encoding='utf-8')
            else:
                f = open("%s/%s.%s" % (path, file_name, postfix), encoding='utf-8')
            res = f.read()
            return res
    except Exception as e:
        if silence is True:
            pass
        else:
            showlog.warning(e)
        return


def read_(_source_file):
    # >>读取数据【方式：一次性全部读取】------------------------------------------------------------
    data = xlrd.open_workbook(_source_file)  # 打开表
    res = list()
    # table = data.sheets()[0]  # 默认使用第一张表格
    for table in data.sheets():
        nrows = table.nrows  # 获取行数
        ncols = table.ncols  # 获取列数
        for inrows in range(nrows):
            res_temp = dict()
            for incols in range(ncols):
                res_temp.update({table.cell(0, incols).value: table.cell(inrows, incols).value})
            # print(res_temp)
            res.append(res_temp)
    return res


def excel_read(source_file):
    # >>读取数据【方式：一次性全部读取】------------------------------------------------------------
    data = xlrd.open_workbook(source_file)  # 打开表
    res = list()
    # table = data.sheets()[0]  # 默认使用第一张表格
    for table in data.sheets():
        nrows = table.nrows  # 获取行数
        ncols = table.ncols  # 获取列数
        for inrows in range(nrows):
            res_temp = dict()
            for incols in range(ncols):
                res_temp.update({table.cell(0, incols).value: table.cell(inrows, incols).value})
            # print(res_temp)
            res.append(res_temp)
    return res


def excel_read_all_table(source_file):
    try:
        # >>读取数据【方式：一次性全部读取】------------------------------------------------------------
        data = xlrd.open_workbook(source_file)  # 打开表
        res = list()
        # table = data.sheets()[0]  # 默认使用第一张表格
        for table in data.sheets():
            nrows = table.nrows  # 获取行数
            ncols = table.ncols  # 获取列数
            for inrows in range(nrows):
                res_temp = dict()
                for incols in range(ncols):
                    res_temp.update({table.cell(0, incols).value: table.cell(inrows, incols).value})
                # print(res_temp)
                res.append(res_temp)
        return res
    except:
        return


def excel_sheet_dict(source_file):
    try:
        # >>读取数据【方式：一次性全部读取】------------------------------------------------------------
        data = xlrd.open_workbook(source_file)  # 打开表
        res = dict()
        for table in data.sheets():
            table_name = table.name
            nrows = table.nrows  # 获取行数
            ncols = table.ncols  # 获取列数
            res_temp = list()
            for inrows in range(nrows):
                data_temp = dict()
                for incols in range(ncols):
                    data_temp.update({table.cell(0, incols).value: table.cell(inrows, incols).value})
                res_temp.append(data_temp)
            res.update({table_name: res_temp})
        return res
    except:
        return


def excel_specify_dict(source_file, sheet_name, _up_data_col_list=None, _col_dict=None):
    try:
        col_dict = dict()
        if _col_dict is not None:
            temp0 = _col_dict.replace(" ", "").split(",")
            if temp0 is not None and len(temp0) > 0:
                for each in temp0:
                    temp1 = each.split(":")
                    col_dict.update({temp1[1]: temp1[0]})
        up_data_col_list = list()
        if _up_data_col_list is not None:
            up_data_col_list = _up_data_col_list.replace(" ", "").split(",")

        # >>读取数据【方式：一次性全部读取】------------------------------------------------------------
        data = xlrd.open_workbook(source_file)  # 打开表
        res = dict()
        for table in data.sheets():
            table_name = table.name
            if table_name == sheet_name:
                nrows = table.nrows  # 获取行数
                ncols = table.ncols  # 获取列数
                res_temp = list()
                for inrows in range(nrows):
                    data_temp = dict()
                    for incols in range(ncols):
                        # print(table.cell(0, incols).value)
                        col_name_ori = table.cell(0, incols).value
                        if len(col_dict) > 0 and col_dict is not None:
                            col_name = col_dict.get(col_name_ori)
                        else:
                            col_name = col_name_ori

                        if col_name_ori in up_data_col_list:
                            # print(col_name_ori)
                            try:
                                col_value = xlrd.xldate.xldate_as_datetime(table.cell(inrows, incols).value, 0)
                            except Exception as ex:
                                # print(ex)
                                col_value = None
                        else:
                            col_value = table.cell(inrows, incols).value

                        data_temp.update({col_name: col_value})
                    res_temp.append(data_temp)
                res.update({table_name: res_temp})
        return res
    except:
        return


def read_txt(text_name, path=None):
    try:
        if path is None:
            f = open("%s.txt" %text_name, encoding='utf-8')
        else:
            f = open("%s/%s.txt" % (path, text_name), encoding='utf-8')
        res = f.read()
        return res
    except:
        return


def read_file(file_name, suffix_name, path=None):
    try:
        if path is None:
            f = open("\%s.%s" % (file_name, suffix_name))
        else:
            f = open("\%s\%s.%s" % (path, file_name, suffix_name))
        res = f.read()
        return res
    except:
        return


def dir_file_list(file_dir):
    file_list = list()
    for root, dirs, files in os.walk(file_dir):
        # print(root)  # 当前目录路径
        # print(dirs)  # 当前路径下所有子目录
        # print(files)  # 当前路径下所有非目录子文件
        file_list.extend(files)
    return file_list


def get_file_size(file_dir):
    # 获取文件大小
    size_byte = os.path.getsize(file_dir)  # 字节
    size_kb = int(size_byte / 1024)
    size_mb = round(size_byte / (1024 * 1024), 2)
    size_gb = round(size_byte / (1024 * 1024 * 1024), 2)
    if size_gb < 1:
        if size_mb < 1:
            size_str = str(size_kb) + ' KB'
        else:
            size_str = str(size_mb) + ' MB'
    else:
        size_str = str(size_gb) + ' GB'
    temp_dict = {
        'size_str': size_str,
        'size_byte': size_byte
    }
    return temp_dict


class DateEncoder(json.JSONEncoder):
    # 处理json.dumps中会出现的datetime格式无法转换问题：json.dumps(each, cls=DateEncoder)
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        else:
            return json.JSONEncoder.default(self, obj)


def save(
        file_name,
        content,
        postfix: str = "txt",
        path=None,
        overwrite=True
):
    """
    增加自动创建目录功能
    """
    if overwrite is True:
        write_mode = "w"  # 覆盖
    else:
        write_mode = "a"  # 追加

    if path is None:
        f = open("%s.%s" % (file_name, postfix), write_mode, encoding='utf-8')
    else:
        lazypath.make_path(path)
        f = open("%s%s%s.%s" % (path, path_separator, file_name, postfix), write_mode, encoding='utf-8')
    f.write(content)
    f.close()


def save_list(
        file_name,
        list_data,
        split_by='\n',
        postfix="txt",
        path=None,
        overwrite=False
):
    """
    将list按照行存储到文件中，对于datetime类型自动转换为日期时间的格式
    """
    if path is None:
        file_dir = "%s.%s" % (file_name, postfix)
    else:
        lazypath.make_path(path)
        file_dir = "%s%s%s.%s" % (path, path_separator, file_name, postfix)

    if overwrite is True:

        write_mode = "w"  # 覆盖
    else:
        write_mode = "a"  # 追加

    f = open(file_dir, write_mode, encoding='utf-8')
    for each in list_data:

        if isinstance(each, collections.OrderedDict):
            f.write(str(json.dumps(each, cls=DateEncoder)))
        else:
            f.write(str(each))
        f.write(split_by)
    f.close()
    return file_dir


def dict_write07excel(path, table_name, sheet_name, data):
    wb = openpyxl.Workbook()
    sheet = wb.active
    sheet.title = sheet_name
    if len(data) > 0:
        row_num = 1
        col_num = 1
        for key in data[0]:
            sheet.cell(row=row_num, column=col_num, value=key)
            col_num += 1
        row_num += 1

        for each in data:
            col_num = 1
            for key in each:
                sheet.cell(row=row_num, column=col_num, value=each.get(key))
                col_num += 1
            row_num += 1

        wb.save("%s/%s.xlsx" % (path, table_name))
        showlog.info("导出数据成功！共计%s条数据" % len(data))


def save_txt(
        text_name,
        content,
        path=None,
        overwrite=True):
    """
    是否覆写看overwrite参数
    设置自动创建目录
    """
    if overwrite is True:
        mode = "w"
    else:
        mode = "a"

    if path is None:
        f = open("%s.txt" % text_name, mode, encoding='utf-8')
    else:
        lazypath.make_path(path)
        f = open("%s%s%s.txt" % (path, path_separator, text_name), mode, encoding='utf-8')
    f.write(content)
    f.close()


def save_sql(
        file_name,
        content,
        path=None
):
    """
    如果文件存在，先清空，再保存
    :param file_name:
    :param content:
    :param path:
    :return:
    """
    if path is None:
        f = open(file_name, "w", encoding='utf-8')
    else:
        f = open("%s%s%s.sql" % (path, path_separator, file_name), "w", encoding='utf-8')
    f.write(content)
    f.close()



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
from . import lazyprocess
from . import lazyrequests
from . import lazybase64
from . import lazytime
from . import lazywebhook
from . import lazyrandom
from . import lazyip
from . import lazyflask
from . import lazyid




#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
from multiprocessing import Process
from multiprocessing import Queue
import showlog
import time
import copy
import os
os_cpu_count = os.cpu_count()  # CPU核心数


def run(
        task_list: list,
        task_function,
        subprocess_keep: bool = False,
        subprocess_limit: int = os_cpu_count * 2,
        master_process_delay: int = 1,
        return_data: bool = False,

        silence: bool = False
):
    """
    多进程 进程控制
    :param task_list: 任务列表，list格式
    :param task_function: 子任务的function，需提前写好，入参为：(task_index, task_info)，例如：task_function(task_index, task_info)
    :param subprocess_keep: 是否保持子进程，True为保持进程，死掉会自动重启；False为不保持，自然退出
    :param subprocess_limit: 进程数限制，0为无限制，否则按照设定的数量限制并行的子进程数量
    :param master_process_delay: 主进程循环延时，单位为秒，默认为1秒
    :param return_data: 是否返回数据，True返回，False不返回
    :param silence: 静默模式，为True是不产生任何提示

    demo:
    def task_function(
            task_index,
            task_info
    ):
        # 进程详细的内容
        print(task_index, task_info)
    """
    active_process = dict()  # 活跃进程，存放进程，以task_index为key，进程信息为value的dict
    task_count = len(task_list)  # 总任务数量
    task_index_start = 0  # 用来计算启动的累计进程数
    q = Queue()  # 生成一个队列对象，以实现进程通信

    showlog.info(f'正在准备多进程执行任务，总任务数为：{task_count}，进程数限制为：{subprocess_limit}...')
    # 创建并启动线程
    while True:
        this_time_start = copy.deepcopy(task_index_start)  # 确定本次循环的起点任务序号
        for task_index in range(this_time_start, task_count):  # 按照任务量遍历
            if task_index in active_process.keys():
                # 进程已存在，不重复创建，跳过
                continue
            else:
                # 进程不存在，待定
                if subprocess_limit > 0:
                    # 存在子进程数量限制规则，待定
                    if len(active_process) >= subprocess_limit:
                        # 当前活跃进程数量达到子进程数限制，本次循环不再新增进程，跳出
                        if silence is False:
                            showlog.warning('达到进程数限制')
                        break
                # 不存在子进程限制规则/当前活跃进程数量未达到进程数限制，将开启一个新进程
                if silence is False:
                    showlog.info(f'发现需要开启的进程：{task_index}')
                task_info = task_list[task_index]  # 提取将开启的进程的任务内容
                # ---------- 开启进程 ----------
                if return_data is True:
                    p = Process(
                        target=task_function,
                        args=(task_index, task_info, q)
                    )
                else:
                    p = Process(
                        target=task_function,
                        args=(task_index, task_info)
                    )
                p.start()
                # ---------- 开启进程 ----------
                active_process[task_index] = {
                    'task_index': task_index,  # 任务序号
                    'task_info': task_info,  # 任务详情
                    'process': p,  # 进程对象
                }  # 记录开启的进程
                if silence is False:
                    showlog.info(f'进程：{task_index} 已开启')
                task_index_start += 1  # 记录累计开启进程数

        # 检测非活跃进程，并从active_process中剔除非活跃进程，以便开启新的进程
        inactive_process = list()  # 非活跃进程
        for process_index, process_info in active_process.items():
            # print(q.qsize())
            # print(q.get())
            # print(q.get_nowait())
            if process_info['process'].is_alive() is False:
                if silence is False:
                    showlog.warning(f'进程 {process_index} 不活跃，将被剔除...')
                inactive_process.append(process_index)
        for each_dead_process in inactive_process:
            active_process.pop(each_dead_process)

        if task_index_start >= len(task_list) and len(active_process) == 0:
            if silence is False:
                showlog.info('全部任务执行完成')
            if subprocess_keep is True:
                task_index_start = 0  # 将累计启动进程数重置为0
            else:
                return
        time.sleep(master_process_delay)



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import requests
import showlog
import time


def lazy_requests(
        method,
        url,
        retry_delay: int = 1,  # 重试延时
        retry_limit: int = -1,  # 重试次数限制，-1为无限制
        return_json: bool = True,  # 是否返回json数据

        ReadTimeout_retry: bool = True,  # 超时重试
        JSONDecodeError_retry: bool = True,  # 返回非json类型重试
        ConnectionError_retry: bool = True,  # 连接错误重试

        **kwargs
):
    """
    retry_delay: 重试延时时长，单位为秒，默认为1秒

    return_json: 返回json，为True时返回json，否则直接返回

    ReadTimeout_retry: 超时重试，为True会自动重试，否则不会自动重试
    JSONDecodeError_retry：返回类型为非json重试，为True会自动重试，否则不会自动重试
    """
    retry_count = 0
    while True:
        try:
            if return_json is True:
                return requests.request(
                    method=method,
                    url=url,
                    **kwargs
                ).json()
            else:
                return requests.request(
                    method=method,
                    url=url,
                    **kwargs
                )
        except requests.exceptions.ReadTimeout:
            if ReadTimeout_retry is True:
                retry_count += 1
                showlog.warning(f'ReadTimeout_retry，将在{retry_delay}秒后重试第{retry_count}次...')
                time.sleep(retry_delay)
        except requests.exceptions.JSONDecodeError:
            if JSONDecodeError_retry is True:
                retry_count += 1
                showlog.warning(f'JSONDecodeError_retry，将在{retry_delay}秒后重试第{retry_count}次...')
                time.sleep(retry_delay)
        except requests.exceptions.ConnectionError:  # 包含ProxyError
            if ConnectionError_retry is True:
                retry_count += 1
                showlog.warning(f'ConnectionError_retry，将在{retry_delay}秒后重试第{retry_count}次...')
                time.sleep(retry_delay)
        if retry_limit == 0:
            break
        elif retry_limit < 0:
            pass
        else:
            if retry_count > retry_limit:
                break
            else:
                pass
